@@ -1,20 +1,27 @@
-/************************************************
+/**********************************************************************
 
   dln.c -
 
   $Author: matz $
-  $Date: 2000/06/05 09:00:52 $
+  $Date: 2000/08/07 05:01:47 $
   created at: Tue Jan 18 17:05:06 JST 1994
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
 
-************************************************/
+**********************************************************************/
 
 #include "config.h"
 #include "defines.h"
 #include "dln.h"
 
+#ifdef __CHECKER__
+#undef HAVE_DLOPEN
+#undef USE_DLN_A_OUT
+#undef USE_DLN_DLOPEN
+#endif
+
 char *dln_argv0;
+void rb_loaderror();
 
 #ifdef _AIX
 #pragma alloca
@@ -37,11 +44,7 @@ void *xrealloc();
 #endif
 
 #include <stdio.h>
-#ifndef NT
-# ifndef USE_CWGUSI
-#  include <sys/file.h>
-# endif
-#else
+#ifdef NT
 #include "missing/file.h"
 #endif
 #include <sys/types.h>
@@ -74,7 +77,7 @@ char *getenv();
 
 int eaccess();
 
-#if defined(HAVE_DLOPEN) && !defined(USE_DLN_A_OUT) && !defined(__CYGWIN32__) && !defined(_AIX)
+#if defined(HAVE_DLOPEN) && !defined(USE_DLN_A_OUT) && !defined(_AIX)
 /* dynamic load with dlopen() */
 # define USE_DLN_DLOPEN
 #endif
@@ -125,16 +128,15 @@ init_funcname(buf, file)
 static int dln_errno;
 
 #define DLN_ENOEXEC	ENOEXEC	/* Exec format error */
-#define DLN_ECONFL	201	/* Symbol name conflict */
-#define DLN_ENOINIT	202	/* No inititalizer given */
-#define DLN_EUNDEF	203	/* Undefine symbol remains */
-#define DLN_ENOTLIB	204	/* Not a library file */
-#define DLN_EBADLIB	205	/* Malformed library file */
-#define DLN_EINIT	206	/* Not initialized */
+#define DLN_ECONFL	1201	/* Symbol name conflict */
+#define DLN_ENOINIT	1202	/* No inititalizer given */
+#define DLN_EUNDEF	1203	/* Undefine symbol remains */
+#define DLN_ENOTLIB	1204	/* Not a library file */
+#define DLN_EBADLIB	1205	/* Malformed library file */
+#define DLN_EINIT	1206	/* Not initialized */
 
 static int dln_init_p = 0;
 
-#include "st.h"
 #include <ar.h>
 #include <a.out.h>
 #ifndef N_COMM
@@ -146,6 +148,9 @@ static int dln_init_p = 0;
 
 #define INVALID_OBJECT(h) (N_MAGIC(h) != OMAGIC)
 
+#include "util.h"
+#include "st.h"
+
 static st_table *sym_tbl;
 static st_table *undef_tbl;
 
@@ -613,7 +618,7 @@ load_1(fd, disp, need_init)
     struct nlist *sym;
     struct nlist *end;
     int init_p = 0;
-    char buf[256];
+    char buf[MAXPATHLEN];
 
     if (load_header(fd, &hdr, disp) == -1) return -1;
     if (INVALID_OBJECT(hdr)) {
@@ -1106,7 +1111,7 @@ dln_sym(name)
 #endif
 
 
-#ifdef _WIN32
+#if defined _WIN32 && !defined __CYGWIN__
 #include <windows.h>
 #endif
 
@@ -1138,7 +1143,7 @@ dln_strerror()
     return (char*)dlerror();
 #endif
 
-#ifdef _WIN32
+#if defined _WIN32 && !defined __CYGWIN__
     static char message[1024];
     int error = GetLastError();
     char *p = message;
@@ -1213,12 +1218,14 @@ void
 dln_load(file)
     const char *file;
 {
-#ifdef _WIN32
+#if defined _WIN32 && !defined __CYGWIN__
     HINSTANCE handle;
-    char winfile[255];
+    char winfile[MAXPATHLEN];
     void (*init_fct)();
     char buf[MAXPATHLEN];
 
+    if (strlen(file) >= MAXPATHLEN) rb_loaderror("filename too long");
+
     /* Load the file as an object one */
     init_funcname(buf, file);
 
@@ -1424,11 +1431,12 @@ dln_load(file)
 				  B_SYMBOL_TYPE_TEXT, (void **)&init_fct);
 
       if (err_stat != B_NO_ERROR) {
-	  char real_name[MAXPATHLEN];
-	  strcpy(real_name, buf);
-	  strcat(real_name, "__Fv");
-	  err_stat = get_image_symbol(img_id, real_name,
-				      B_SYMBOL_TYPE_TEXT, (void **)&init_fct);
+	char real_name[MAXPATHLEN];
+
+	strcpy(real_name, buf);
+	strcat(real_name, "__Fv");
+        err_stat = get_image_symbol(img_id, real_name,
+				    B_SYMBOL_TYPE_TEXT, (void **)&init_fct);
       }
 
       if ((B_BAD_IMAGE_ID == err_stat) || (B_BAD_INDEX == err_stat)) {
