@@ -1,14 +1,14 @@
-/************************************************
+/**********************************************************************
 
   error.c -
 
   $Author: matz $
-  $Date: 2000/05/01 09:42:21 $
+  $Date: 2000/08/24 06:21:28 $
   created at: Mon Aug  9 16:11:34 JST 1993
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
 
-************************************************/
+**********************************************************************/
 
 #include "ruby.h"
 #include "env.h"
@@ -21,11 +21,6 @@
 #define va_init_list(a,b) va_start(a)
 #endif
 
-#ifdef USE_CWGUSI
-#include <sys/errno.h>
-int sys_nerr = 256;
-#endif
-
 #if defined __CYGWIN__
 # include <cygwin/version.h>
 # if (CYGWIN_VERSION_API_MAJOR > 0) || (CYGWIN_VERSION_API_MINOR >= 8)
@@ -164,6 +159,7 @@ rb_bug(fmt, va_alist)
     va_init_list(args, fmt);
     err_print(buf, args);
     va_end(args);
+    ruby_show_version();
     abort();
 }
 
@@ -185,8 +181,10 @@ static struct types {
     T_STRUCT,	"Struct",
     T_BIGNUM,	"Bignum",
     T_FILE,	"File",
-    T_TRUE,	"TRUE",
-    T_FALSE,	"FALSE",
+    T_TRUE,	"true",
+    T_FALSE,	"false",
+    T_UNDEF,	"Symbol",	/* :symbol */
+    T_UNDEF,	"undef",	/* internal use: #undef */
     T_DATA,	"Data",		/* internal use: wrapped C pointers */
     T_MATCH,	"Match",	/* data of $~ */
     T_VARMAP,	"Varmap",	/* internal use: dynamic variables */
@@ -239,14 +237,18 @@ VALUE rb_eSignal;
 VALUE rb_eFatal;
 VALUE rb_eStandardError;
 VALUE rb_eRuntimeError;
-VALUE rb_eSyntaxError;
 VALUE rb_eTypeError;
 VALUE rb_eArgError;
-VALUE rb_eNameError;
 VALUE rb_eIndexError;
-VALUE rb_eLoadError;
+VALUE rb_eRangeError;
 VALUE rb_eSecurityError;
 VALUE rb_eNotImpError;
+VALUE rb_eNoMemError;
+
+VALUE rb_eScriptError;
+VALUE rb_eNameError;
+VALUE rb_eSyntaxError;
+VALUE rb_eLoadError;
 
 VALUE rb_eSystemCallError;
 VALUE rb_mErrno;
@@ -278,7 +280,7 @@ rb_exc_new3(etype, str)
     char *s;
     int len;
 
-    s = str2cstr(str, &len);
+    s = rb_str2cstr(str, &len);
     return rb_exc_new(etype, s, len);
 }
 
@@ -325,6 +327,7 @@ exc_to_s(exc)
     VALUE mesg = rb_iv_get(exc, "mesg");
 
     if (NIL_P(mesg)) return rb_class_path(CLASS_OF(exc));
+    if (OBJ_TAINTED(exc)) OBJ_TAINT(mesg);
     return mesg;
 }
 
@@ -342,9 +345,9 @@ exc_inspect(exc)
 
     str = rb_str_new2("#<");
     klass = rb_class_path(klass);
-    rb_str_concat(str, klass);
+    rb_str_append(str, klass);
     rb_str_cat(str, ": ", 2);
-    rb_str_concat(str, exc);
+    rb_str_append(str, exc);
     rb_str_cat(str, ">", 1);
 
     return str;
@@ -538,16 +541,22 @@ Init_Exception()
     rb_eSignal      = rb_define_class("SignalException", rb_eException);
 
     rb_eStandardError = rb_define_class("StandardError", rb_eException);
-    rb_eSyntaxError = rb_define_class("SyntaxError", rb_eStandardError);
     rb_eTypeError   = rb_define_class("TypeError", rb_eStandardError);
     rb_eArgError    = rb_define_class("ArgumentError", rb_eStandardError);
-    rb_eNameError   = rb_define_class("NameError", rb_eStandardError);
     rb_eIndexError  = rb_define_class("IndexError", rb_eStandardError);
-    rb_eLoadError   = rb_define_class("LoadError", rb_eStandardError);
+    rb_eRangeError  = rb_define_class("RangeError", rb_eStandardError);
+
+    rb_eScriptError = rb_define_class("ScriptError", rb_eException);
+    rb_eSyntaxError = rb_define_class("SyntaxError", rb_eScriptError);
+    rb_eNameError   = rb_define_class("NameError", rb_eScriptError);
+    rb_eLoadError   = rb_define_class("LoadError", rb_eScriptError);
+    rb_eNotImpError = rb_define_class("NotImplementedError", rb_eScriptError);
+    /* backward compatibility -- will be removed in the future */
+    rb_define_global_const("NotImplementError", rb_eNotImpError);
 
     rb_eRuntimeError = rb_define_class("RuntimeError", rb_eStandardError);
     rb_eSecurityError = rb_define_class("SecurityError", rb_eStandardError);
-    rb_eNotImpError = rb_define_class("NotImplementError", rb_eException);
+    rb_eNoMemError = rb_define_class("NoMemoryError", rb_eException);
 
     init_syserr();
 }
@@ -632,8 +641,10 @@ rb_sys_fail(mesg)
 
     err = strerror(errno);
     if (mesg) {
-	buf = ALLOCA_N(char, strlen(err)+strlen(mesg)+4);
-	sprintf(buf, "%s - %s", err, mesg);
+	volatile VALUE tmp = rb_str_inspect(rb_str_new2(mesg));
+
+	buf = ALLOCA_N(char, strlen(err)+RSTRING(tmp)->len+4);
+	sprintf(buf, "%s - %s", err, RSTRING(tmp)->ptr);
     }
     else {
 	buf = ALLOCA_N(char, strlen(err)+1);
@@ -650,17 +661,6 @@ rb_sys_fail(mesg)
 	ee = set_syserr(n, name);
    }
 #else
-# ifdef USE_CWGUSI
-    if (n < 0) {
-	int macoserr_index = sys_nerr - 1;
-	if (!syserr_list[macoserr_index]) {
-	    char name[6];
-	    sprintf(name, "E%03d", macoserr_index);
-	    ee = set_syserr(macoserr_index, name);
-	}
-    }
-    else
-#endif /* USE_CWGUSI */
     if (n > sys_nerr || !syserr_list[n]) {
 	char name[6];
 
@@ -676,6 +676,13 @@ rb_sys_fail(mesg)
     rb_exc_raise(ee);
 }
 
+void
+rb_error_frozen(what)
+    char *what;
+{
+    rb_raise(rb_eTypeError, "can't modify frozen %s", what);
+}
+
 static void
 init_syserr()
 {
@@ -1080,8 +1087,8 @@ err_append(s)
 	else {
 	    VALUE str = rb_str_to_str(ruby_errinfo);
 
-	    rb_str_cat(str, "\n", 1);
-	    rb_str_cat(str, s, strlen(s));
+	    rb_str_cat2(str, "\n");
+	    rb_str_cat2(str, s);
 	    ruby_errinfo = rb_exc_new3(rb_eSyntaxError, str);
 	}
     }
