@@ -8,6 +8,17 @@
 #include <stdio.h>
 #endif
 
+#if defined(_MSC_VER)
+#if defined(_M_ALPHA)
+#ifdef __cplusplus
+extern "C" { long __asm(char *,...); };
+#else
+long __asm(char *,...);
+#endif
+#pragma intrinsic(__asm)
+#endif
+#endif
+
 #define _T_VOID     0
 #define _T_NUMBER   1
 #define _T_POINTER  2
@@ -48,10 +59,13 @@ Win32API_initialize(self, dllname, proc, import, export)
     VALUE str;
     VALUE a_import;
     VALUE *ptr;
+    char *s;
     int i;
     int len;
     int ex;
 
+    Check_SafeStr(dllname);
+    Check_SafeStr(proc);
     hdll = LoadLibrary(RSTRING(dllname)->ptr);
     if (!hdll)
 	rb_raise(rb_eRuntimeError, "LoadLibrary: %s\n", RSTRING(dllname)->ptr);
@@ -70,36 +84,64 @@ Win32API_initialize(self, dllname, proc, import, export)
     rb_iv_set(self, "__proc__", UINT2NUM((unsigned long)hproc));
 
     a_import = rb_ary_new();
-    ptr = RARRAY(import)->ptr;
-    for (i = 0, len = RARRAY(import)->len; i < len; i++) {
-	int c = *(char *)RSTRING(ptr[i])->ptr;
-	switch (c) {
-	case 'N': case 'n': case 'L': case 'l':
-	    rb_ary_push(a_import, INT2FIX(_T_NUMBER));
-	    break;
-	case 'P': case 'p':
-	    rb_ary_push(a_import, INT2FIX(_T_POINTER));
-	    break;
-	case 'I': case 'i':
-	    rb_ary_push(a_import, INT2FIX(_T_INTEGER));
-	    break;
+    switch (TYPE(import)) {
+      case T_NIL:
+	break;
+      case T_ARRAY:
+	ptr = RARRAY(import)->ptr;
+	for (i = 0, len = RARRAY(import)->len; i < len; i++) {
+	    Check_SafeStr(ptr[i]);
+	    switch (*(char *)RSTRING(ptr[i])->ptr) {
+	      case 'N': case 'n': case 'L': case 'l':
+		rb_ary_push(a_import, INT2FIX(_T_NUMBER));
+		break;
+	      case 'P': case 'p':
+		rb_ary_push(a_import, INT2FIX(_T_POINTER));
+		break;
+	      case 'I': case 'i':
+		rb_ary_push(a_import, INT2FIX(_T_INTEGER));
+		break;
+	    }
 	}
+        break;
+      default:
+	Check_SafeStr(import);
+	s = RSTRING(import)->ptr;
+	for (i = 0, len = RSTRING(import)->len; i < len; i++) {
+	    switch (*s++) {
+	      case 'N': case 'n': case 'L': case 'l':
+		rb_ary_push(a_import, INT2FIX(_T_NUMBER));
+		break;
+	      case 'P': case 'p':
+		rb_ary_push(a_import, INT2FIX(_T_POINTER));
+		break;
+	      case 'I': case 'i':
+		rb_ary_push(a_import, INT2FIX(_T_INTEGER));
+		break;
+	    }
+	}
+        break;
     }
     rb_iv_set(self, "__import__", a_import);
 
-    switch (*RSTRING(export)->ptr) {
-    case 'V': case 'v':
+    if (NIL_P(export)) {
 	ex = _T_VOID;
-	break;
-    case 'N': case 'n': case 'L': case 'l':
-	ex = _T_NUMBER;
-	break;
-    case 'P': case 'p':
-	ex = _T_POINTER;
-	break;
-    case 'I': case 'i':
-	ex = _T_INTEGER;
-	break;
+    } else {
+	Check_SafeStr(export);
+	switch (*RSTRING(export)->ptr) {
+	  case 'V': case 'v':
+	    ex = _T_VOID;
+	    break;
+	  case 'N': case 'n': case 'L': case 'l':
+	    ex = _T_NUMBER;
+	    break;
+	  case 'P': case 'p':
+	    ex = _T_POINTER;
+	    break;
+	  case 'I': case 'i':
+	    ex = _T_INTEGER;
+	    break;
+	}
     }
     rb_iv_set(self, "__export__", INT2FIX(ex));
 
@@ -159,10 +201,20 @@ Win32API_Call(argc, argv, obj)
 	    case _T_INTEGER:
 		lParam = NUM2ULONG(rb_ary_entry(args, i));
 #if defined(_MSC_VER) || defined(__LCC__)
+#if defined(_M_IX86)
 		_asm {
 		    mov     eax, lParam
 		    push    eax
 		}
+#elif defined(_M_ALPHA)
+		__asm(
+			"ldl r0, 0(%0);"
+			"stq r0, -(sp);"
+			, lParam
+		);
+#else
+#error
+#endif
 #elif defined __GNUC__
 		asm volatile ("pushl %0" :: "g" (lParam));
 #else
@@ -181,10 +233,20 @@ Win32API_Call(argc, argv, obj)
 		    pParam = RSTRING(str)->ptr;
 		}
 #if defined(_MSC_VER) || defined(__LCC__)
+#if defined(_M_IX86)
 		_asm {
 		    mov     eax, pParam
 		    push    eax
 		}
+#elif defined(_M_ALPHA)
+		__asm(
+			"ldl r0, 0(%0);"
+			"stq r0, -(sp);"
+			, pParam
+		);
+#else
+#error
+#endif
 #elif defined __GNUC__
 		asm volatile ("pushl %0" :: "g" (pParam));
 #else
