@@ -3,7 +3,7 @@
   dbm.c -
 
   $Author: matz $
-  $Date: 2000/06/20 06:08:49 $
+  $Date: 2000/09/12 05:37:33 $
   created at: Mon Jan 24 15:59:52 JST 1994
 
   Copyright (C) 1995-1998 Yukihiro Matsumoto
@@ -15,12 +15,12 @@
 #ifdef HAVE_CDEFS_H
 # include <cdefs.h>
 #endif
+#ifdef HAVE_SYS_CDEFS_H
+# include <sys/cdefs.h>
+#endif
 #include <ndbm.h>
 #include <fcntl.h>
 #include <errno.h>
-#ifdef USE_CWGUSI
-# include <sys/errno.h>
-#endif
 
 VALUE cDBM;
 
@@ -72,12 +72,16 @@ fdbm_s_open(argc, argv, klass)
     Check_SafeStr(file);
 
     dbm = 0;
-    if (mode >= 0)
+    if (mode >= 0) {
 	dbm = dbm_open(RSTRING(file)->ptr, O_RDWR|O_CREAT, mode);
-    if (!dbm)
+    }
+    if (!dbm) {
+	mode = 0666;
 	dbm = dbm_open(RSTRING(file)->ptr, O_RDWR, mode);
-    if (!dbm)
+    }
+    if (!dbm) {
 	dbm = dbm_open(RSTRING(file)->ptr, O_RDONLY, mode);
+    }
 
     if (!dbm) {
 	if (mode == -1) return Qnil;
@@ -87,7 +91,6 @@ fdbm_s_open(argc, argv, klass)
     obj = Data_Make_Struct(klass,struct dbmdata,0,free_dbm,dbmp);
     dbmp->di_dbm = dbm;
     dbmp->di_size = -1;
-    rb_obj_call_init(obj, argc, argv);
 
     return obj;
 }
@@ -106,8 +109,8 @@ fdbm_close(obj)
 }
 
 static VALUE
-fdbm_fetch(obj, keystr)
-    VALUE obj, keystr;
+fdbm_fetch(obj, keystr, ifnone)
+    VALUE obj, keystr, ifnone;
 {
     datum key, value;
     struct dbmdata *dbmp;
@@ -121,11 +124,55 @@ fdbm_fetch(obj, keystr)
     dbm = dbmp->di_dbm;
     value = dbm_fetch(dbm, key);
     if (value.dptr == 0) {
-	return Qnil;
+	if (ifnone == Qnil && rb_block_given_p())
+	    return rb_yield(rb_tainted_str_new(key.dptr, key.dsize));
+	return ifnone;
     }
     return rb_tainted_str_new(value.dptr, value.dsize);
 }
 
+static VALUE
+fdbm_aref(obj, keystr)
+    VALUE obj, keystr;
+{
+    return fdbm_fetch(obj, keystr, Qnil);
+}
+
+static VALUE
+fdbm_fetch_m(argc, argv, obj)
+    int argc;
+    VALUE *argv;
+    VALUE obj;
+{
+    VALUE keystr, ifnone;
+
+    rb_scan_args(argc, argv, "11", &keystr, &ifnone);
+    return fdbm_fetch(obj, keystr, ifnone);
+}
+
+static VALUE
+fdbm_index(obj, valstr)
+    VALUE obj, valstr;
+{
+    datum key, val;
+    struct dbmdata *dbmp;
+    DBM *dbm;
+
+    Check_Type(valstr, T_STRING);
+    val.dptr = RSTRING(valstr)->ptr;
+    val.dsize = RSTRING(valstr)->len;
+
+    GetDBM(obj, dbmp);
+    dbm = dbmp->di_dbm;
+    for (key = dbm_firstkey(dbm); key.dptr; key = dbm_nextkey(dbm)) {
+	val = dbm_fetch(dbm, key);
+	if (val.dsize == RSTRING(valstr)->len &&
+	    memcmp(val.dptr, RSTRING(valstr)->ptr, val.dsize) == 0)
+	    return rb_tainted_str_new(key.dptr, key.dsize);
+    }
+    return Qnil;
+}
+
 static VALUE
 fdbm_indexes(argc, argv, obj)
     int argc;
@@ -161,7 +208,7 @@ fdbm_delete(obj, keystr)
 
     value = dbm_fetch(dbm, key);
     if (value.dptr == 0) {
-	if (rb_iterator_p()) rb_yield(keystr);
+	if (rb_block_given_p()) rb_yield(keystr);
 	return Qnil;
     }
 
@@ -310,19 +357,12 @@ fdbm_store(obj, keystr, valstr)
     struct dbmdata *dbmp;
     DBM *dbm;
 
-    if (valstr == Qnil) {
-	fdbm_delete(obj, keystr);
-	return Qnil;
-    }
-
     rb_secure(4);
     keystr = rb_obj_as_string(keystr);
 
     key.dptr = RSTRING(keystr)->ptr;
     key.dsize = RSTRING(keystr)->len;
 
-    if (NIL_P(valstr)) return fdbm_delete(obj, keystr);
-
     valstr = rb_obj_as_string(valstr);
     val.dptr = RSTRING(valstr)->ptr;
     val.dsize = RSTRING(valstr)->len;
@@ -546,6 +586,35 @@ fdbm_to_a(obj)
     return ary;
 }
 
+static VALUE
+fdbm_to_hash(obj)
+    VALUE obj;
+{
+    datum key, val;
+    struct dbmdata *dbmp;
+    DBM *dbm;
+    VALUE hash;
+
+    GetDBM(obj, dbmp);
+    dbm = dbmp->di_dbm;
+
+    hash = rb_hash_new();
+    for (key = dbm_firstkey(dbm); key.dptr; key = dbm_nextkey(dbm)) {
+	val = dbm_fetch(dbm, key);
+	rb_hash_aset(hash, rb_tainted_str_new(key.dptr, key.dsize),
+		           rb_tainted_str_new(val.dptr, val.dsize));
+    }
+
+    return hash;
+}
+
+static VALUE
+fdbm_reject(obj)
+    VALUE obj;
+{
+    return rb_hash_delete_if(fdbm_to_hash(obj));
+}
+
 void
 Init_dbm()
 {
@@ -555,8 +624,11 @@ Init_dbm()
     rb_define_singleton_method(cDBM, "open", fdbm_s_open, -1);
     rb_define_singleton_method(cDBM, "new", fdbm_s_open, -1);
     rb_define_method(cDBM, "close", fdbm_close, 0);
-    rb_define_method(cDBM, "[]", fdbm_fetch, 1);
+    rb_define_method(cDBM, "[]", fdbm_aref, 1);
+    rb_define_method(cDBM, "fetch", fdbm_fetch_m, -1);
     rb_define_method(cDBM, "[]=", fdbm_store, 2);
+    rb_define_method(cDBM, "store", fdbm_store, 2);
+    rb_define_method(cDBM, "index",  fdbm_index, 1);
     rb_define_method(cDBM, "indexes",  fdbm_indexes, -1);
     rb_define_method(cDBM, "indices",  fdbm_indexes, -1);
     rb_define_method(cDBM, "length", fdbm_length, 0);
@@ -571,6 +643,8 @@ Init_dbm()
     rb_define_method(cDBM, "shift", fdbm_shift, 1);
     rb_define_method(cDBM, "delete", fdbm_delete, 1);
     rb_define_method(cDBM, "delete_if", fdbm_delete_if, 0);
+    rb_define_method(cDBM, "reject!", fdbm_delete_if, 0);
+    rb_define_method(cDBM, "reject", fdbm_reject, 0);
     rb_define_method(cDBM, "clear", fdbm_clear, 0);
     rb_define_method(cDBM,"invert", fdbm_invert, 0);
     rb_define_method(cDBM,"update", fdbm_update, 1);
@@ -578,9 +652,11 @@ Init_dbm()
 
     rb_define_method(cDBM, "include?", fdbm_has_key, 1);
     rb_define_method(cDBM, "has_key?", fdbm_has_key, 1);
+    rb_define_method(cDBM, "member?", fdbm_has_key, 1);
     rb_define_method(cDBM, "has_value?", fdbm_has_value, 1);
     rb_define_method(cDBM, "key?", fdbm_has_key, 1);
     rb_define_method(cDBM, "value?", fdbm_has_value, 1);
 
     rb_define_method(cDBM, "to_a", fdbm_to_a, 0);
+    rb_define_method(cDBM, "to_hash", fdbm_to_hash, 0);
 }
