@@ -3,7 +3,7 @@
   socket.c -
 
   $Author: matz $
-  $Date: 2000/07/11 07:55:47 $
+  $Date: 2000/08/24 06:21:42 $
   created at: Thu Mar 31 12:21:29 JST 1994
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
@@ -21,7 +21,11 @@
 #endif
 
 #ifndef NT
-#include <sys/socket.h>
+#if defined(__BEOS__)
+# include <net/socket.h>
+#else
+# include <sys/socket.h>
+#endif
 #include <netinet/in.h>
 #ifdef HAVE_NETINET_TCP_H
 # include <netinet/tcp.h>
@@ -381,7 +385,7 @@ bsock_send(argc, argv, sock)
   retry:
     rb_thread_fd_writable(fd);
     m = rb_str2cstr(msg, &mlen);
-    if (RTEST(to)) {
+    if (!NIL_P(to)) {
 	t = rb_str2cstr(to, &tlen);
 	n = sendto(fd, m, mlen, NUM2INT(flags),
 		   (struct sockaddr*)t, tlen);
@@ -413,14 +417,13 @@ static VALUE unixaddr _((struct sockaddr_un *));
 
 enum sock_recv_type {
     RECV_RECV,			/* BasicSocket#recv(no from) */
-    RECV_TCP,			/* TCPSocket#recvfrom */
-    RECV_UDP,			/* UDPSocket#recvfrom */
+    RECV_IP,			/* IPSocket#recvfrom */
     RECV_UNIX,			/* UNIXSocket#recvfrom */
     RECV_SOCKET,		/* Socket#recvfrom */
 };
 
 static VALUE
-s_recv(sock, argc, argv, from)
+s_recvfrom(sock, argc, argv, from)
     VALUE sock;
     int argc;
     VALUE *argv;
@@ -439,6 +442,9 @@ s_recv(sock, argc, argv, from)
     else             flags = NUM2INT(flg);
 
     GetOpenFile(sock, fptr);
+    if (rb_read_pending(fptr->f)) {
+	rb_raise(rb_eIOError, "recv for buffered IO");
+    }
     fd = fileno(fptr->f);
 
     str = rb_tainted_str_new(0, NUM2INT(len));
@@ -469,8 +475,7 @@ s_recv(sock, argc, argv, from)
     switch (from) {
       case RECV_RECV:
 	return (VALUE)str;
-      case RECV_TCP:
-      case RECV_UDP:
+      case RECV_IP:
 #if 0
 	if (alen != sizeof(struct sockaddr_in)) {
 	    rb_raise(rb_eTypeError, "sockaddr size differs - should not happen");
@@ -492,7 +497,7 @@ bsock_recv(argc, argv, sock)
     VALUE *argv;
     VALUE sock;
 {
-    return s_recv(sock, argc, argv, RECV_RECV);
+    return s_recvfrom(sock, argc, argv, RECV_RECV);
 }
 
 static VALUE
@@ -632,6 +637,9 @@ ipaddr(sockaddr)
     char hbuf[1024], pbuf[1024];
 
     switch (sockaddr->sa_family) {
+    case AF_UNSPEC:
+	family = rb_str_new2("AF_UNSPEC");
+	break;
     case AF_INET:
 	family = rb_str_new2("AF_INET");
 	break;
@@ -639,9 +647,19 @@ ipaddr(sockaddr)
     case AF_INET6:
 	family = rb_str_new2("AF_INET6");
 	break;
+#endif
+#ifdef AF_LOCAL
+    case AF_LOCAL:
+	family = rb_str_new2("AF_LOCAL");
+	break;
+#elif  AF_UNIX
+    case AF_UNIX:
+	family = rb_str_new2("AF_UNIX");
+	break;
 #endif
     default:
-	family = 0;
+        sprintf(pbuf, "unknown:%d", sockaddr->sa_family);
+	family = rb_str_new2(pbuf);
 	break;
     }
     if (!do_not_reverse_lookup) {
@@ -1026,6 +1044,7 @@ s_accept(class, fd, sockaddr, len)
     socklen_t *len;
 {
     int fd2;
+    int retry = 0;
 
     rb_secure(3);
   retry:
@@ -1035,6 +1054,12 @@ s_accept(class, fd, sockaddr, len)
     TRAP_END;
     if (fd2 < 0) {
 	switch (errno) {
+	  case EMFILE:
+	  case ENFILE:
+	    if (retry) break;
+	    rb_gc();
+	    retry = 1;
+	    goto retry;
 	  case EINTR:
 	    rb_thread_schedule();
 	    goto retry;
@@ -1065,15 +1090,6 @@ tcp_accept(sock)
 		    (struct sockaddr*)&from, &fromlen);
 }
 
-static VALUE
-tcp_recvfrom(argc, argv, sock)
-    int argc;
-    VALUE *argv;
-    VALUE sock;
-{
-    return s_recv(sock, argc, argv, RECV_TCP);
-}
-
 #ifdef HAVE_SYS_UN_H
 static VALUE
 open_unix(class, path, server)
@@ -1153,6 +1169,15 @@ ip_peeraddr(sock)
     return ipaddr((struct sockaddr *)&addr);
 }
 
+static VALUE
+ip_recvfrom(argc, argv, sock)
+    int argc;
+    VALUE *argv;
+    VALUE sock;
+{
+    return s_recvfrom(sock, argc, argv, RECV_IP);
+}
+
 static VALUE
 ip_s_getaddress(obj, host)
     VALUE obj, host;
@@ -1245,7 +1270,7 @@ udp_send(argc, argv, sock)
     int mlen;
     struct addrinfo *res0, *res;
 
-    if (argc == 2) {
+    if (argc == 2 || argc == 3) {
 	return bsock_send(argc, argv, sock);
     }
     rb_secure(4);
@@ -1281,15 +1306,6 @@ udp_send(argc, argv, sock)
     return INT2FIX(n);
 }
 
-static VALUE
-udp_recvfrom(argc, argv, sock)
-    int argc;
-    VALUE *argv;
-    VALUE sock;
-{
-    return s_recv(sock, argc, argv, RECV_UDP);
-}
-
 #ifdef HAVE_SYS_UN_H
 static VALUE
 unix_s_sock_open(sock, path)
@@ -1328,7 +1344,7 @@ unix_recvfrom(argc, argv, sock)
     VALUE *argv;
     VALUE sock;
 {
-    return s_recv(sock, argc, argv, RECV_UNIX);
+    return s_recvfrom(sock, argc, argv, RECV_UNIX);
 }
 
 static VALUE
@@ -1571,7 +1587,7 @@ sock_recvfrom(argc, argv, sock)
     VALUE *argv;
     VALUE sock;
 {
-    return s_recv(sock, argc, argv, RECV_SOCKET);
+    return s_recvfrom(sock, argc, argv, RECV_SOCKET);
 }
 
 static VALUE
@@ -1825,12 +1841,21 @@ sock_s_getaddrinfo(argc, argv)
     }
 
     MEMZERO(&hints, struct addrinfo, 1);
-    if (!NIL_P(family)) {
-	hints.ai_family = NUM2INT(family);
-    }
-    else {
+    if (NIL_P(family)) {
 	hints.ai_family = PF_UNSPEC;
     }
+    else if (FIXNUM_P(family)) {
+	hints.ai_family = FIX2INT(family);
+    }
+    else if (strcmp(STR2CSTR(family), "AF_INET") == 0) {
+	hints.ai_family = PF_INET;
+    }
+#ifdef INET6
+    else if (strcmp(STR2CSTR(family), "AF_INET6") == 0) {
+	hints.ai_family = PF_INET6;
+    }
+#endif
+
     if (!NIL_P(socktype)) {
 	hints.ai_socktype = NUM2INT(socktype);
     }
@@ -2023,6 +2048,7 @@ Init_socket()
     rb_define_global_const("IPsocket", rb_cIPSocket);
     rb_define_method(rb_cIPSocket, "addr", ip_addr, 0);
     rb_define_method(rb_cIPSocket, "peeraddr", ip_peeraddr, 0);
+    rb_define_method(rb_cIPSocket, "recvfrom", ip_recvfrom, -1);
     rb_define_singleton_method(rb_cIPSocket, "getaddress", ip_s_getaddress, 1);
 
     rb_cTCPSocket = rb_define_class("TCPSocket", rb_cIPSocket);
@@ -2030,7 +2056,6 @@ Init_socket()
     rb_define_singleton_method(rb_cTCPSocket, "open", tcp_s_open, 2);
     rb_define_singleton_method(rb_cTCPSocket, "new", tcp_s_open, 2);
     rb_define_singleton_method(rb_cTCPSocket, "gethostbyname", tcp_s_gethostbyname, 1);
-    rb_define_method(rb_cTCPSocket, "recvfrom", tcp_recvfrom, -1);
 
 #ifdef SOCKS
     rb_cSOCKSSocket = rb_define_class("SOCKSSocket", rb_cTCPSocket);
@@ -2055,7 +2080,6 @@ Init_socket()
     rb_define_method(rb_cUDPSocket, "connect", udp_connect, 2);
     rb_define_method(rb_cUDPSocket, "bind", udp_bind, 2);
     rb_define_method(rb_cUDPSocket, "send", udp_send, -1);
-    rb_define_method(rb_cUDPSocket, "recvfrom", udp_recvfrom, -1);
 
 #ifdef HAVE_SYS_UN_H
     rb_cUNIXSocket = rb_define_class("UNIXSocket", rb_cBasicSocket);
