@@ -3,7 +3,7 @@
   gc.c -
 
   $Author: matz $
-  $Date: 2000/09/12 05:37:21 $
+  $Date: 2000/09/27 03:43:13 $
   created at: Tue Oct  5 09:44:46 JST 1993
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
@@ -41,9 +41,9 @@ static void run_final();
 
 #ifndef GC_MALLOC_LIMIT
 #if defined(MSDOS) || defined(__human68k__)
-#define GC_MALLOC_LIMIT 100000
+#define GC_MALLOC_LIMIT 200000
 #else
-#define GC_MALLOC_LIMIT 4000000
+#define GC_MALLOC_LIMIT 8000000
 #endif
 #endif
 
@@ -70,6 +70,7 @@ ruby_xmalloc(size)
     }
     if (size == 0) size = 1;
     malloc_memories += size;
+
     if (malloc_memories > GC_MALLOC_LIMIT) {
 	rb_gc();
     }
@@ -118,7 +119,7 @@ ruby_xrealloc(ptr, size)
 	rb_gc();
 	mem = realloc(ptr, size);
 	if (!mem)
-	    if (size >= 10 * 1024 * 1024) {
+	    if (size >= 50 * 1024 * 1024) {
 		rb_raise(rb_eNoMemError, "tryed to re-allocate too big memory");
 	    }
 	    mem_error("failed to allocate memory(realloc)");
@@ -752,19 +753,27 @@ obj_free(obj)
 	break;
       case T_STRING:
 #define STR_NO_ORIG FL_USER2	/* copied from string.c */
-	if (!RANY(obj)->as.string.orig || FL_TEST(obj, STR_NO_ORIG))
+	if (!RANY(obj)->as.string.orig || FL_TEST(obj, STR_NO_ORIG)) {
 	    free(RANY(obj)->as.string.ptr);
+	}
 	break;
       case T_ARRAY:
-	if (RANY(obj)->as.array.ptr) free(RANY(obj)->as.array.ptr);
+	if (RANY(obj)->as.array.ptr) {
+	    free(RANY(obj)->as.array.ptr);
+	}
 	break;
       case T_HASH:
-	if (RANY(obj)->as.hash.tbl)
+	if (RANY(obj)->as.hash.tbl) {
 	    st_free_table(RANY(obj)->as.hash.tbl);
+	}
 	break;
       case T_REGEXP:
-	if (RANY(obj)->as.regexp.ptr) re_free_pattern(RANY(obj)->as.regexp.ptr);
-	if (RANY(obj)->as.regexp.str) free(RANY(obj)->as.regexp.str);
+	if (RANY(obj)->as.regexp.ptr) {
+	    re_free_pattern(RANY(obj)->as.regexp.ptr);
+	}
+	if (RANY(obj)->as.regexp.str) {
+	    free(RANY(obj)->as.regexp.str);
+	}
 	break;
       case T_DATA:
 	if (DATA_PTR(obj)) {
@@ -797,7 +806,9 @@ obj_free(obj)
 	break;
 
       case T_BIGNUM:
-	if (RANY(obj)->as.bignum.digits) free(RANY(obj)->as.bignum.digits);
+	if (RANY(obj)->as.bignum.digits) {
+	    free(RANY(obj)->as.bignum.digits);
+	}
 	break;
       case T_NODE:
 	switch (nd_type(obj)) {
@@ -826,8 +837,9 @@ obj_free(obj)
 	break;
 
       case T_STRUCT:
-	if (RANY(obj)->as.rstruct.ptr)
+	if (RANY(obj)->as.rstruct.ptr) {
 	    free(RANY(obj)->as.rstruct.ptr);
+	}
 	break;
 
       default:
@@ -1298,4 +1310,3 @@ xfree(ptr)
 {
     ruby_xfree(ptr);
 }
-
