@@ -1,41 +1,30 @@
-/************************************************
+/**********************************************************************
 
   hash.c -
 
   $Author: matz $
-  $Date: 2000/07/24 07:16:07 $
+  $Date: 2000/09/12 05:37:23 $
   created at: Mon Nov 22 18:51:18 JST 1993
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
+  Copyright (C) 2000  Network Applied Communication Laboratory, Inc.
+  Copyright (C) 2000  Information-technology Promotion Agency, Japan
 
-************************************************/
+**********************************************************************/
 
 #include "ruby.h"
 #include "st.h"
 #include "util.h"
 #include "rubysig.h"
 
-#include <sys/types.h>
-#include <sys/stat.h>
-
-#ifndef HAVE_STRING_H
-char *strchr _((char*,char));
-#endif
-
-#ifndef strdup
-char *strdup();
-#endif
-
-#define HASH_FREEZE   FL_USER1
-#define HASH_DELETED  FL_USER2
+#define HASH_DELETED  FL_USER1
 
 static void
 rb_hash_modify(hash)
     VALUE hash;
 {
-    if (FL_TEST(hash, HASH_FREEZE))
-	rb_raise(rb_eTypeError, "can't modify frozen hash");
-    if (!FL_TEST(hash, FL_TAINT) && rb_safe_level() >= 4)
+    if (OBJ_FROZEN(hash)) rb_error_frozen("hash");
+    if (!OBJ_TAINTED(hash) && rb_safe_level() >= 4)
 	rb_raise(rb_eSecurityError, "Insecure: can't modify hash");
 }
 
@@ -43,17 +32,7 @@ VALUE
 rb_hash_freeze(hash)
     VALUE hash;
 {
-    FL_SET(hash, HASH_FREEZE);
-    return hash;
-}
-
-static VALUE
-rb_hash_frozen_p(hash)
-    VALUE hash;
-{
-    if (FL_TEST(hash, HASH_FREEZE))
-	return Qtrue;
-    return Qfalse;
+    return rb_obj_freeze(hash);
 }
 
 VALUE rb_cHash;
@@ -139,7 +118,7 @@ rb_hash_foreach_iter(key, value, arg)
     st_table *tbl = RHASH(arg->hash)->tbl;
     struct st_table_entry **bins = tbl->bins;
 
-    if (value == Qnil) return ST_CONTINUE;
+    if (key == Qundef) return ST_CONTINUE;
     status = (*arg->func)(key, value, arg->arg);
     if (RHASH(arg->hash)->tbl != tbl || RHASH(arg->hash)->tbl->bins != bins){
 	rb_raise(rb_eIndexError, "rehash occurred during iteration");
@@ -163,7 +142,7 @@ rb_hash_foreach_ensure(hash)
 
     if (RHASH(hash)->iter_lev == 0) {
 	if (FL_TEST(hash, HASH_DELETED)) {
-	    st_cleanup_safe(RHASH(hash)->tbl, Qnil);
+	    st_cleanup_safe(RHASH(hash)->tbl, Qundef);
 	    FL_UNSET(hash, HASH_DELETED);
 	}
     }
@@ -185,29 +164,6 @@ rb_hash_foreach(hash, func, farg)
     return rb_ensure(rb_hash_foreach_call, (VALUE)&arg, rb_hash_foreach_ensure, hash);
 }
 
-static VALUE
-rb_hash_s_new(argc, argv, klass)
-    int argc;
-    VALUE *argv;
-    VALUE klass;
-{
-    VALUE ifnone;
-
-    NEWOBJ(hash, struct RHash);
-    OBJSETUP(hash, klass, T_HASH);
-
-    hash->iter_lev = 0;
-    hash->ifnone = Qnil;
-    hash->tbl = 0;		/* avoid GC crashing  */
-
-    rb_scan_args(argc, argv, "01", &ifnone);
-
-    hash->ifnone = ifnone;
-    hash->tbl = st_init_table(&objhash);
-
-    return (VALUE)hash;
-}
-
 static VALUE
 rb_hash_new2(klass)
     VALUE klass;
@@ -215,9 +171,7 @@ rb_hash_new2(klass)
     NEWOBJ(hash, struct RHash);
     OBJSETUP(hash, klass, T_HASH);
 
-    hash->iter_lev = 0;
     hash->ifnone = Qnil;
-    hash->tbl = 0;		/* avoid GC crashing  */
     hash->tbl = st_init_table(&objhash);
 
     return (VALUE)hash;
@@ -229,6 +183,33 @@ rb_hash_new()
     return rb_hash_new2(rb_cHash);
 }
 
+static VALUE
+rb_hash_s_new(argc, argv, klass)
+    int argc;
+    VALUE *argv;
+    VALUE klass;
+{
+    VALUE hash = rb_hash_new2(klass);
+
+    rb_obj_call_init(hash, argc, argv);
+    return hash;
+}
+
+static VALUE
+rb_hash_initialize(argc, argv, hash)
+    int argc;
+    VALUE *argv;
+    VALUE hash;
+{
+    VALUE ifnone;
+
+    rb_scan_args(argc, argv, "01", &ifnone);
+    rb_hash_modify(hash);
+    RHASH(hash)->ifnone = ifnone;
+
+    return hash;
+}
+
 static VALUE
 rb_hash_s_create(argc, argv, klass)
     int argc;
@@ -242,9 +223,7 @@ rb_hash_s_create(argc, argv, klass)
 	NEWOBJ(hash, struct RHash);
 	OBJSETUP(hash, klass, T_HASH);
 	    
-	hash->iter_lev = 0;
 	hash->ifnone = Qnil;
-	hash->tbl = 0;	/* avoid GC crashing  */
 	hash->tbl = st_copy(RHASH(argv[0])->tbl);
 
 	return (VALUE)hash;
@@ -256,7 +235,6 @@ rb_hash_s_create(argc, argv, klass)
     hash = rb_hash_new2(klass);
 
     for (i=0; i<argc; i+=2) {
-	if (NIL_P(argv[i+1])) continue;
 	st_insert(RHASH(hash)->tbl, argv[i], argv[i+1]);
     }
 
@@ -267,30 +245,13 @@ static VALUE
 rb_hash_clone(hash)
     VALUE hash;
 {
-    NEWOBJ(hash2, struct RHash);
-    CLONESETUP(hash2, hash);
+    NEWOBJ(clone, struct RHash);
+    CLONESETUP(clone, hash);
 
-    hash2->iter_lev = 0;
-    hash2->ifnone = RHASH(hash)->ifnone;
-    hash2->tbl = 0;		/* avoid GC crashing  */
-    hash2->tbl = (st_table*)st_copy(RHASH(hash)->tbl);
+    clone->ifnone = RHASH(hash)->ifnone;
+    clone->tbl = (st_table*)st_copy(RHASH(hash)->tbl);
 
-    return (VALUE)hash2;
-}
-
-static VALUE
-rb_hash_dup(hash)
-    VALUE hash;
-{
-    NEWOBJ(hash2, struct RHash);
-    OBJSETUP(hash2, CLASS_OF(hash), T_HASH);
-
-    hash2->iter_lev = 0;
-    hash2->ifnone = RHASH(hash)->ifnone;
-    hash2->tbl = 0;		/* avoid GC crashing  */
-    hash2->tbl = (st_table*)st_copy(RHASH(hash)->tbl);
-
-    return (VALUE)hash2;
+    return (VALUE)clone;
 }
 
 static VALUE
@@ -305,9 +266,7 @@ rb_hash_rehash_i(key, value, tbl)
     VALUE key, value;
     st_table *tbl;
 {
-    if (key != Qnil) {
-	st_insert(tbl, key, value);
-    }
+    if (key != Qundef) st_insert(tbl, key, value);
     return ST_CONTINUE;
 }
 
@@ -349,11 +308,14 @@ rb_hash_fetch(argc, argv, hash)
     rb_scan_args(argc, argv, "11", &key, &if_none);
 
     if (!st_lookup(RHASH(hash)->tbl, key, &val)) {
-	if (rb_iterator_p()) {
+	if (rb_block_given_p()) {
 	    if (argc > 1) {
 		rb_raise(rb_eArgError, "wrong # of arguments", argc);
 	    }
-	    return rb_yield(argv[0]);
+	    return rb_yield(key);
+	}
+	if (argc == 1) {
+	    rb_raise(rb_eIndexError, "key not found");
 	}
 	return if_none;
     }
@@ -394,7 +356,7 @@ rb_hash_index(hash, value)
     VALUE args[2];
 
     args[0] = value;
-    args[1] = Qnil;
+    args[1] = RHASH(hash)->ifnone;
 
     st_foreach(RHASH(hash)->tbl, index_i, args);
 
@@ -426,17 +388,17 @@ rb_hash_delete(hash, key)
 
     rb_hash_modify(hash);
     if (RHASH(hash)->iter_lev > 0) {
-	if (st_delete_safe(RHASH(hash)->tbl, &key, &val, Qnil)) {
+	if (st_delete_safe(RHASH(hash)->tbl, &key, &val, Qundef)) {
 	    FL_SET(hash, HASH_DELETED);
 	    return val;
 	}
     }
     else if (st_delete(RHASH(hash)->tbl, &key, &val))
 	return val;
-    if (rb_iterator_p()) {
+    if (rb_block_given_p()) {
 	return rb_yield(key);
     }
-    return Qnil;
+    return RHASH(hash)->ifnone;
 }
 
 struct shift_var {
@@ -450,7 +412,7 @@ shift_i(key, value, var)
     VALUE key, value;
     struct shift_var *var;
 {
-    if (value == Qnil) return ST_CONTINUE;
+    if (key == Qundef) return ST_CONTINUE;
     if (var->stop) return ST_STOP;
     var->stop = 1;
     var->key = key;
@@ -468,7 +430,7 @@ rb_hash_shift(hash)
     var.stop = 0;
     st_foreach(RHASH(hash)->tbl, shift_i, &var);
 
-    if (var.stop == 0) return Qnil;
+    if (var.stop == 0) return RHASH(hash)->ifnone;
     return rb_assoc_new(var.key, var.val);
 }
 
@@ -476,22 +438,38 @@ static int
 delete_if_i(key, value)
     VALUE key, value;
 {
-    if (value == Qnil) return ST_CONTINUE;
+    if (key == Qundef) return ST_CONTINUE;
     if (RTEST(rb_yield(rb_assoc_new(key, value))))
 	return ST_DELETE;
     return ST_CONTINUE;
 }
 
-static VALUE
+VALUE
 rb_hash_delete_if(hash)
     VALUE hash;
 {
     rb_hash_modify(hash);
     rb_hash_foreach(hash, delete_if_i, 0);
-
     return hash;
 }
 
+VALUE
+rb_hash_reject_bang(hash)
+    VALUE hash;
+{
+    int n = RHASH(hash)->tbl->num_entries;
+    rb_hash_delete_if(hash);
+    if (n == RHASH(hash)->tbl->num_entries) return Qnil;
+    return hash;
+}
+
+static VALUE
+rb_hash_reject(hash)
+    VALUE hash;
+{
+    return rb_hash_delete_if(rb_obj_dup(hash));
+}
+
 static int
 clear_i(key, value, dummy)
     VALUE key, value, dummy;
@@ -514,10 +492,6 @@ rb_hash_aset(hash, key, val)
     VALUE hash, key, val;
 {
     rb_hash_modify(hash);
-    if (NIL_P(val)) {
-	rb_hash_delete(hash, key);
-	return Qnil;
-    }
     if (TYPE(key) != T_STRING || st_lookup(RHASH(hash)->tbl, key, 0)) {
 	st_insert(RHASH(hash)->tbl, key, val);
     }
@@ -547,7 +521,7 @@ rb_hash_replace(hash, hash2)
 }
 
 static VALUE
-rb_hash_length(hash)
+rb_hash_size(hash)
     VALUE hash;
 {
     return INT2FIX(RHASH(hash)->tbl->num_entries);
@@ -566,7 +540,7 @@ static int
 each_value_i(key, value)
     VALUE key, value;
 {
-    if (value == Qnil) return ST_CONTINUE;
+    if (key == Qundef) return ST_CONTINUE;
     rb_yield(value);
     return ST_CONTINUE;
 }
@@ -583,7 +557,7 @@ static int
 each_key_i(key, value)
     VALUE key, value;
 {
-    if (value == Qnil) return ST_CONTINUE;
+    if (key == Qundef) return ST_CONTINUE;
     rb_yield(key);
     return ST_CONTINUE;
 }
@@ -600,7 +574,7 @@ static int
 each_pair_i(key, value)
     VALUE key, value;
 {
-    if (value == Qnil) return ST_CONTINUE;
+    if (key == Qundef) return ST_CONTINUE;
     rb_yield(rb_assoc_new(key, value));
     return ST_CONTINUE;
 }
@@ -617,7 +591,7 @@ static int
 to_a_i(key, value, ary)
     VALUE key, value, ary;
 {
-    if (value == Qnil) return ST_CONTINUE;
+    if (key == Qundef) return ST_CONTINUE;
     rb_ary_push(ary, rb_assoc_new(key, value));
     return ST_CONTINUE;
 }
@@ -630,6 +604,7 @@ rb_hash_to_a(hash)
 
     ary = rb_ary_new();
     st_foreach(RHASH(hash)->tbl, to_a_i, ary);
+    if (OBJ_TAINTED(hash)) OBJ_TAINT(ary);
 
     return ary;
 }
@@ -638,7 +613,9 @@ static VALUE
 rb_hash_sort(hash)
     VALUE hash;
 {
-    return rb_ary_sort_bang(rb_hash_to_a(hash));
+    VALUE entries = rb_hash_to_a(hash);
+    rb_ary_sort_bang(entries);
+    return entries;
 }
 
 static int
@@ -647,15 +624,17 @@ inspect_i(key, value, str)
 {
     VALUE str2;
 
-    if (value == Qnil) return ST_CONTINUE;
+    if (key == Qundef) return ST_CONTINUE;
     if (RSTRING(str)->len > 1) {
-	rb_str_cat(str, ", ", 2);
+	rb_str_cat2(str, ", ");
     }
     str2 = rb_inspect(key);
-    rb_str_cat(str, RSTRING(str2)->ptr, RSTRING(str2)->len);
-    rb_str_cat(str, "=>", 2);
+    rb_str_append(str, str2);
+    OBJ_INFECT(str, str2);
+    rb_str_cat2(str, "=>");
     str2 = rb_inspect(value);
-    rb_str_cat(str, RSTRING(str2)->ptr, RSTRING(str2)->len);
+    rb_str_append(str, str2);
+    OBJ_INFECT(str, str2);
 
     return ST_CONTINUE;
 }
@@ -668,8 +647,9 @@ inspect_hash(hash)
 
     str = rb_str_new2("{");
     st_foreach(RHASH(hash)->tbl, inspect_i, str);
-    rb_str_cat(str, "}", 1);
-
+    rb_str_cat2(str, "}");
+    
+    OBJ_INFECT(str, hash);
     return str;
 }
 
@@ -693,8 +673,6 @@ static VALUE
 rb_hash_to_s(hash)
     VALUE hash;
 {
-    VALUE str;
-
     if (rb_inspecting_p(hash)) return rb_str_new2("{...}");
     return rb_protect_inspect(to_s_hash, hash, 0);
 }
@@ -710,7 +688,7 @@ static int
 keys_i(key, value, ary)
     VALUE key, value, ary;
 {
-    if (value == Qnil) return ST_CONTINUE;
+    if (key == Qundef) return ST_CONTINUE;
     rb_ary_push(ary, key);
     return ST_CONTINUE;
 }
@@ -731,7 +709,7 @@ static int
 values_i(key, value, ary)
     VALUE key, value, ary;
 {
-    if (value == Qnil) return ST_CONTINUE;
+    if (key == Qundef) return ST_CONTINUE;
     rb_ary_push(ary, value);
     return ST_CONTINUE;
 }
@@ -763,7 +741,7 @@ static int
 rb_hash_search_value(key, value, data)
     VALUE key, value, *data;
 {
-    if (value == Qnil) return ST_CONTINUE;
+    if (key == Qundef) return ST_CONTINUE;
     if (rb_equal(value, data[1])) {
 	data[0] = Qtrue;
 	return ST_STOP;
@@ -796,7 +774,7 @@ equal_i(key, val1, data)
 {
     VALUE val2;
 
-    if (val1 == Qnil) return ST_CONTINUE;
+    if (key == Qundef) return ST_CONTINUE;
     if (!st_lookup(data->tbl, key, &val2)) {
 	data->result = Qfalse;
 	return ST_STOP;
@@ -830,7 +808,7 @@ rb_hash_invert_i(key, value, hash)
     VALUE key, value;
     VALUE hash;
 {
-    if (value == Qnil) return ST_CONTINUE;
+    if (key == Qundef) return ST_CONTINUE;
     rb_hash_aset(hash, value, key);
     return ST_CONTINUE;
 }
@@ -850,7 +828,7 @@ rb_hash_update_i(key, value, hash)
     VALUE key, value;
     VALUE hash;
 {
-    if (value == Qnil) return ST_CONTINUE;
+    if (key == Qundef) return ST_CONTINUE;
     rb_hash_aset(hash, key, value);
     return ST_CONTINUE;
 }
@@ -879,27 +857,29 @@ env_delete(obj, name)
     char *nam, *val;
 
     rb_secure(4);
-    nam = str2cstr(name, &len);
+    nam = rb_str2cstr(name, &len);
     if (strlen(nam) != len) {
 	rb_raise(rb_eArgError, "bad environment variable name");
     }
     val = getenv(nam);
     if (val) {
+	VALUE value = rb_tainted_str_new2(val);
+
 	ruby_setenv(nam, 0);
 	if (strcmp(nam, "PATH") == 0 && !OBJ_TAINTED(name)) {
 	    path_tainted = 0;
 	}
-	return rb_str_new2(val);
+	return value;
     }
     return Qnil;
 }
 
 static VALUE
-env_delete_method(obj, name)
+env_delete_m(obj, name)
     VALUE obj, name;
 {
     VALUE val = env_delete(obj, name);
-    if (rb_iterator_p()) rb_yield(name);
+    if (rb_block_given_p()) rb_yield(name);
     return val;
 }
 
@@ -910,7 +890,7 @@ rb_f_getenv(obj, name)
     char *nam, *env;
     int len;
 
-    nam = str2cstr(name, &len);
+    nam = rb_str2cstr(name, &len);
     if (strlen(nam) != len) {
 	rb_raise(rb_eArgError, "bad environment variable name");
     }
@@ -923,52 +903,36 @@ rb_f_getenv(obj, name)
     return Qnil;
 }
 
-static int
-path_check_1(path)
-    char *path;
+static VALUE
+env_fetch(argc, argv)
+    int argc;
+    VALUE *argv;
 {
-    struct stat st;
-    char *p = 0;
-    char *s;
+    VALUE key, if_none;
+    char *nam, *env;
+    int len;
 
-    for (;;) {
-	if (stat(path, &st) == 0 && (st.st_mode & 002)) {
-	    return 0;
-	}
-	s = strrchr(path, '/');
-	if (p) *p = '/';
-	if (!s || s == path) return 1;
-	p = s;
-	*p = '\0';
+    rb_scan_args(argc, argv, "11", &key, &if_none);
+    nam = rb_str2cstr(key, &len);
+    if (strlen(nam) != len) {
+	rb_raise(rb_eArgError, "bad environment variable name");
     }
-}
-
-int
-rb_path_check(path)
-    char *path;
-{
-    char *p, *pend;
-    const char sep = PATH_SEP_CHAR;
-
-    if (!path) return 1;
-
-    p = path;
-    pend = strchr(path, sep);
-    
-    for (;;) {
-	int safe;
-
-	if (pend) *pend = '\0';
-	safe = path_check_1(p);
-	if (!pend) break;
-	*pend = sep;
-	if (!safe) {
-	    return 0;
+    env = getenv(nam);
+    if (!env) {
+	if (rb_block_given_p()) {
+	    if (argc > 1) {
+		rb_raise(rb_eArgError, "wrong # of arguments", argc);
+	    }
+	    return rb_yield(key);
 	}
-	p = pend + 1;
-	pend = strchr(p, sep);
+	if (argc == 1) {
+	    rb_raise(rb_eIndexError, "key not found");
+	}
+	return if_none;
     }
-    return 1;
+    if (strcmp(nam, "PATH") == 0 && !rb_env_path_tainted())
+	return rb_str_new2(env);
+    return rb_tainted_str_new2(env);
 }
 
 static void
@@ -1091,7 +1055,8 @@ ruby_setenv(name, value)
 	environ = tmpenv;		/* tell exec where it is now */
     }
     if (!value) {
-	free(environ[i]);
+	if (environ[i] != origenviron[i])
+	    free(environ[i]);
 	while (environ[i]) {
 	    environ[i] = environ[i+1];
 	    i++;
@@ -1103,7 +1068,8 @@ ruby_setenv(name, value)
 	environ[i+1] = 0;	/* make sure it's null terminated */
     }
     else {
-	free(environ[i]);
+	if (environ[i] != origenviron[i])
+	    free(environ[i]);
     }
     environ[i] = ALLOC_N(char, strlen(name) + strlen(value) + 2);
 #ifndef MSDOS
@@ -1144,8 +1110,8 @@ rb_f_setenv(obj, nm, val)
 	return Qnil;
     }
 
-    name = str2cstr(nm, &nlen);
-    value = str2cstr(val, &vlen);
+    name = rb_str2cstr(nm, &nlen);
+    value = rb_str2cstr(val, &vlen);
     if (strlen(name) != nlen)
 	rb_raise(rb_eArgError, "bad environment variable name");
     if (strlen(value) != vlen)
@@ -1252,11 +1218,11 @@ env_each(hash)
 }
 
 static VALUE
-env_delete_if()
+env_reject_bang()
 {
     volatile VALUE keys;
     VALUE *ptr;
-    int len;
+    int len, del = 0;
 
     rb_secure(4);
     keys = env_keys();
@@ -1268,10 +1234,19 @@ env_delete_if()
 	if (!NIL_P(val)) {
 	    if (RTEST(rb_yield(rb_assoc_new(*ptr, val)))) {
 		env_delete(Qnil, *ptr);
+		del++;
 	    }
 	}
 	ptr++;
     }
+    if (del == 0) return Qnil;
+    return envtbl;
+}
+
+static VALUE
+env_delete_if()
+{
+    env_reject_bang();
     return envtbl;
 }
 
@@ -1396,8 +1371,7 @@ env_indexes(argc, argv)
 }
 
 static VALUE
-env_to_hash(obj)
-    VALUE obj;
+env_to_hash()
 {
     char **env;
     VALUE hash = rb_hash_new();
@@ -1414,6 +1388,12 @@ env_to_hash(obj)
     return hash;
 }
 
+static VALUE
+env_reject()
+{
+    return rb_hash_delete_if(env_to_hash());
+}
+
 void
 Init_Hash()
 {
@@ -1425,14 +1405,11 @@ Init_Hash()
 
     rb_define_singleton_method(rb_cHash, "new", rb_hash_s_new, -1);
     rb_define_singleton_method(rb_cHash, "[]", rb_hash_s_create, -1);
+    rb_define_method(rb_cHash,"initialize", rb_hash_initialize, -1);
 
     rb_define_method(rb_cHash,"clone", rb_hash_clone, 0);
-    rb_define_method(rb_cHash,"dup", rb_hash_dup, 0);
     rb_define_method(rb_cHash,"rehash", rb_hash_rehash, 0);
 
-    rb_define_method(rb_cHash,"freeze", rb_hash_freeze, 0);
-    rb_define_method(rb_cHash,"frozen?",rb_hash_frozen_p, 0);
-
     rb_define_method(rb_cHash,"to_hash", rb_hash_to_hash, 0);
     rb_define_method(rb_cHash,"to_a", rb_hash_to_a, 0);
     rb_define_method(rb_cHash,"to_s", rb_hash_to_s, 0);
@@ -1448,8 +1425,8 @@ Init_Hash()
     rb_define_method(rb_cHash,"index", rb_hash_index, 1);
     rb_define_method(rb_cHash,"indexes", rb_hash_indexes, -1);
     rb_define_method(rb_cHash,"indices", rb_hash_indexes, -1);
-    rb_define_method(rb_cHash,"length", rb_hash_length, 0);
-    rb_define_alias(rb_cHash, "size", "length");
+    rb_define_method(rb_cHash,"size", rb_hash_size, 0);
+    rb_define_method(rb_cHash,"length", rb_hash_size, 0);
     rb_define_method(rb_cHash,"empty?", rb_hash_empty_p, 0);
 
     rb_define_method(rb_cHash,"each", rb_hash_each_pair, 0);
@@ -1464,7 +1441,8 @@ Init_Hash()
     rb_define_method(rb_cHash,"shift", rb_hash_shift, 0);
     rb_define_method(rb_cHash,"delete", rb_hash_delete, 1);
     rb_define_method(rb_cHash,"delete_if", rb_hash_delete_if, 0);
-    rb_define_method(rb_cHash,"reject!", rb_hash_delete_if, 0);
+    rb_define_method(rb_cHash,"reject", rb_hash_reject, 0);
+    rb_define_method(rb_cHash,"reject!", rb_hash_reject_bang, 0);
     rb_define_method(rb_cHash,"clear", rb_hash_clear, 0);
     rb_define_method(rb_cHash,"invert", rb_hash_invert, 0);
     rb_define_method(rb_cHash,"update", rb_hash_update, 1);
@@ -1483,20 +1461,24 @@ Init_Hash()
     rb_extend_object(envtbl, rb_mEnumerable);
 
     rb_define_singleton_method(envtbl,"[]", rb_f_getenv, 1);
+    rb_define_singleton_method(envtbl,"fetch", env_fetch, -1);
     rb_define_singleton_method(envtbl,"[]=", rb_f_setenv, 2);
+    rb_define_singleton_method(envtbl,"store", rb_f_setenv, 2);
     rb_define_singleton_method(envtbl,"each", env_each, 0);
     rb_define_singleton_method(envtbl,"each_pair", env_each, 0);
     rb_define_singleton_method(envtbl,"each_key", env_each_key, 0);
     rb_define_singleton_method(envtbl,"each_value", env_each_value, 0);
-    rb_define_singleton_method(envtbl,"delete", env_delete_method, 1);
+    rb_define_singleton_method(envtbl,"delete", env_delete_m, 1);
     rb_define_singleton_method(envtbl,"delete_if", env_delete_if, 0);
-    rb_define_singleton_method(envtbl,"reject!", env_delete_if, 0);
+    rb_define_singleton_method(envtbl,"reject", env_reject, 0);
+    rb_define_singleton_method(envtbl,"reject!", env_reject_bang, 0);
     rb_define_singleton_method(envtbl,"to_s", env_to_s, 0);
     rb_define_singleton_method(envtbl,"rehash", env_none, 0);
     rb_define_singleton_method(envtbl,"to_a", env_to_a, 0);
     rb_define_singleton_method(envtbl,"index", env_index, 1);
     rb_define_singleton_method(envtbl,"indexes", env_indexes, -1);
     rb_define_singleton_method(envtbl,"indices", env_indexes, -1);
+    rb_define_singleton_method(envtbl,"size", env_size, 0);
     rb_define_singleton_method(envtbl,"length", env_size, 0);
     rb_define_singleton_method(envtbl,"empty?", env_empty_p, 0);
     rb_define_singleton_method(envtbl,"keys", env_keys, 0);
