@@ -1,6 +1,6 @@
-#!./miniruby -I.
+#!./miniruby
 
-require "rbconfig.rb"
+load "./rbconfig.rb"
 include Config
 
 destdir = ARGV[0] || ''
@@ -32,13 +32,19 @@ wdir = Dir.getwd
 File.makedirs bindir, true
 File.install ruby_install_name+exeext,
   "#{bindir}/#{ruby_install_name}#{exeext}", 0755, true
+rubyw = ruby_install_name.sub(/ruby/, '\&w')+exeext
+if File.exist? rubyw
+  File.install rubyw, "#{bindir}/#{rubyw}", 0755, true
+end
 for dll in Dir['*.dll']
   File.install dll, "#{bindir}/#{dll}", 0755, true
 end
 File.makedirs libdir, true
-for lib in ["libruby.so.LIB", CONFIG["LIBRUBY_SO"]]
-  if File.exist? lib
-    File.install lib, libdir, 0555, true
+if CONFIG["LIBRUBY"] != CONFIG["LIBRUBY_A"]
+  for lib in [CONFIG["LIBRUBY"]]
+    if File.exist? lib
+      File.install lib, libdir, 0555, true
+    end
   end
 end
 Dir.chdir libdir
@@ -62,9 +68,15 @@ if RUBY_PLATFORM =~ /-aix/
 end
 
 Dir.chdir "ext"
-system "../miniruby#{exeext} extmk.rb install #{destdir}"
+if defined? CROSS_COMPILING
+  system "#{CONFIG['MINIRUBY']} extmk.rb install #{destdir}"
+else
+  system "../miniruby#{exeext} extmk.rb install #{destdir}"
+end
 Dir.chdir CONFIG["srcdir"]
 
+File.install "sample/irb.rb", "#{bindir}/irb", 0755, true
+
 Find.find("lib") do |f|
   next unless /\.rb$/ =~ f
   dir = rubylibdir+"/"+File.dirname(f[4..-1])
@@ -75,17 +87,14 @@ end
 for f in Dir["*.h"]
   File.install f, archlibdir, 0644, true
 end
-if RUBY_PLATFORM =~ /mswin32/
+if RUBY_PLATFORM =~ /mswin32|mingw/
   File.makedirs archlibdir + "/win32", true
   File.install "win32/win32.h", archlibdir + "/win32", 0644, true
-  if File.exist? wdir+'/rubymw.lib'
-    File.install wdir+'/rubymw.lib', archlibdir, 0644, true
-  end
 end
 File.install wdir+'/'+CONFIG['LIBRUBY_A'], archlibdir, 0644, true
 
 File.makedirs mandir, true
-File.install "ruby.1", mandir, 0644, true
+File.install "ruby.1", mandir+"/"+ruby_install_name+".1", 0644, true
 Dir.chdir wdir
 File.install "config.h", archlibdir, 0644, true
 File.install "rbconfig.rb", archlibdir, 0644, true
