@@ -1,8 +1,8 @@
 #
 #   ruby-lex.rb - ruby lexcal analizer
 #   	$Release Version: 0.6$
-#   	$Revision: 1.20 $
-#   	$Date: 1999/03/11 15:39:55 $
+#   	$Revision: 1.3 $
+#   	$Date: 2000/07/25 06:03:50 $
 #   	by Keiju ISHITSUKA(Nippon Rational Inc.)
 #
 # --
@@ -15,7 +15,7 @@ require "irb/slex"
 require "irb/ruby-token"
 
 class RubyLex
-  @RCS_ID='-$Id: ruby-lex.rb 1.20 1999/03/11 15:39:55 keiju Exp keiju $-'
+  @RCS_ID='-$Id: ruby-lex.rb,v 1.3 2000/07/25 06:03:50 matz Exp $-'
 
   extend Exception2MessageMapper
   def_exception(:AlreadyDefinedToken, "Already defined token(%s)")
@@ -368,8 +368,8 @@ class RubyLex
       if @lex_state != EXPR_END && @lex_state != EXPR_CLASS &&
 	  (@lex_state != EXPR_ARG || @space_seen)
 	c = peek(0)
-	if /\S/ =~ c && (/["'`]/ =~ c || /[\w_]/ =~ c)
-	  tk = identify_here_document;
+	if /\S/ =~ c && (/["'`]/ =~ c || /[\w_]/ =~ c || c == "-")
+	  tk = identify_here_document
 	end
       else
 	tk = 	Token(op)
@@ -752,7 +752,12 @@ class RubyLex
 
   def identify_here_document
     ch = getc
-    if lt = PERCENT_LTYPE[ch]
+    if ch == "-"
+      ch = getc
+      indent = true
+    end
+    if /['"`]/ =~ ch
+      lt = ch
       quoted = ""
       while (c = getc) && c != lt
 	quoted.concat c
@@ -778,7 +783,7 @@ class RubyLex
     end
 
     @here_header = false
-    while (l = gets.chomp) && l != quoted
+    while (l = gets.chomp) && (indent ? l.strip : l) != quoted
     end
 
     @here_header = true
@@ -949,7 +954,7 @@ class RubyLex
 	read_escape(chrs)
       end
     else
-      # other characters 
+      # other characters
     end
   end
 end
