@@ -16,8 +16,13 @@ module Singleton
     klass.instance_eval %{
       @__instance__ = nil
       def instance
+	Thread.critical = true
 	unless @__instance__
-	  @__instance__ = new
+	  begin
+	    @__instance__ = new
+	  ensure
+	    Thread.critical = false
+	  end
 	end
 	return @__instance__
       end
