@@ -1,12 +1,14 @@
-/************************************************
+/**********************************************************************
 
   marshal.c -
 
   $Author: matz $
-  $Date: 2000/06/23 07:05:53 $
+  $Date: 2000/09/19 07:54:21 $
   created at: Thu Apr 27 16:30:01 JST 1995
 
-************************************************/
+  Copyright (C) 1993-2000 Yukihiro Matsumoto
+
+**********************************************************************/
 
 #include "ruby.h"
 #include "rubyio.h"
@@ -17,7 +19,7 @@ double strtod();
 #endif
 
 #define MARSHAL_MAJOR   4
-#define MARSHAL_MINOR   3
+#define MARSHAL_MINOR   4
 
 #define TYPE_NIL	'0'
 #define TYPE_TRUE	'T'
@@ -53,6 +55,7 @@ struct dump_arg {
     VALUE str;
     st_table *symbol;
     st_table *data;
+    int taint;
 };
 
 struct dump_call_arg {
@@ -206,8 +209,6 @@ w_ivar(tbl, arg)
     st_table *tbl;
     struct dump_call_arg *arg;
 {
-    struct dump_call_arg c_arg;
-
     if (tbl) {
 	w_long(tbl->num_entries, arg->arg);
 	st_foreach(tbl, obj_each, arg);
@@ -253,6 +254,10 @@ w_object(obj, arg, limit)
 	}
 #endif
     }
+    else if (SYMBOL_P(obj)) {
+	w_symbol(SYM2ID(obj), arg);
+	return;
+    }
     else {
 	long num;
 
@@ -266,6 +271,8 @@ w_object(obj, arg, limit)
 	    return;
 	}
 
+	if (OBJ_TAINTED(obj)) arg->taint = Qtrue;
+
 	st_add_direct(arg->data, obj, arg->data->num_entries);
 	if (rb_respond_to(obj, s_dump)) {
 	    VALUE v;
@@ -377,10 +384,10 @@ w_object(obj, arg, limit)
 		w_long(len, arg);
 		mem = rb_ivar_get(CLASS_OF(obj), rb_intern("__member__"));
 		if (mem == Qnil) {
-		    rb_raise(rb_eTypeError, "non-initialized struct");
+		    rb_raise(rb_eTypeError, "uninitialized struct");
 		}
 		for (i=0; i<len; i++) {
-		    w_symbol(FIX2LONG(RARRAY(mem)->ptr[i]), arg);
+		    w_symbol(SYM2ID(RARRAY(mem)->ptr[i]), arg);
 		    w_object(RSTRUCT(obj)->ptr[i], arg, limit);
 		}
 	    }
@@ -393,7 +400,9 @@ w_object(obj, arg, limit)
 		char *path;
 
 		if (FL_TEST(klass, FL_SINGLETON)) {
-		    rb_raise(rb_eTypeError, "singleton can't be dumped");
+		    if (RCLASS(klass)->m_tbl->num_entries > 0) {
+			rb_raise(rb_eTypeError, "singleton can't be dumped");
+		    }
 		}
 		path = rb_class2name(klass);
 		w_unique(path, arg);
@@ -426,6 +435,9 @@ dump_ensure(arg)
 {
     st_free_table(arg->symbol);
     st_free_table(arg->data);
+    if (!arg->fp && arg->taint) {
+	OBJ_TAINT(arg->str);
+    }
     return 0;
 }
 
@@ -470,6 +482,7 @@ marshal_dump(argc, argv)
 
     arg.symbol = st_init_numtable();
     arg.data   = st_init_numtable();
+    arg.taint  = Qfalse;
     c_arg.obj = obj;
     c_arg.arg = &arg;
     c_arg.limit = limit;
@@ -488,6 +501,7 @@ struct load_arg {
     st_table *symbol;
     VALUE data;
     VALUE proc;
+    int taint;
 };
 
 static VALUE r_object _((struct load_arg *arg));
@@ -593,20 +607,25 @@ r_bytes0(s, len, arg)
 }
 
 static ID
-r_symbol(arg)
+r_symlink(arg)
+    struct load_arg *arg;
+{
+    ID id;
+    int num = r_long(arg);
+
+    if (st_lookup(arg->symbol, num, &id)) {
+	return id;
+    }
+    rb_raise(rb_eTypeError, "bad symbol");
+}
+
+static ID
+r_symreal(arg)
     struct load_arg *arg;
 {
     char *buf;
     ID id;
 
-    if (r_byte(arg) == TYPE_SYMLINK) {
-	int num = r_long(arg);
-
-	if (st_lookup(arg->symbol, num, &id)) {
-	    return id;
-	}
-	rb_raise(rb_eTypeError, "bad symbol");
-    }
     r_bytes(buf, arg);
     id = rb_intern(buf);
     st_insert(arg->symbol, arg->symbol->num_entries, id);
@@ -614,6 +633,16 @@ r_symbol(arg)
     return id;
 }
 
+static ID
+r_symbol(arg)
+    struct load_arg *arg;
+{
+    if (r_byte(arg) == TYPE_SYMLINK) {
+	return r_symlink(arg);
+    }
+    return r_symreal(arg);
+}
+
 static char*
 r_unique(arg)
     struct load_arg *arg;
@@ -637,11 +666,11 @@ r_regist(v, arg)
     VALUE v;
     struct load_arg *arg;
 {
-    OBJ_TAINT(v);
     if (arg->proc) {
 	rb_funcall(arg->proc, rb_intern("call"), 1, v);
     }
     rb_hash_aset(arg->data, INT2FIX(RHASH(arg->data)->tbl->num_entries), v);
+    if (arg->taint) OBJ_TAINT(v);
     return v;
 }
 
@@ -794,7 +823,7 @@ r_object(arg)
 	    klass = rb_path2class(r_unique(arg));
 	    mem = rb_ivar_get(klass, rb_intern("__member__"));
 	    if (mem == Qnil) {
-		rb_raise(rb_eTypeError, "non-initialized struct");
+		rb_raise(rb_eTypeError, "uninitialized struct");
 	    }
 	    len = r_long(arg);
 
@@ -807,11 +836,11 @@ r_object(arg)
 	    for (i=0; i<len; i++) {
 		slot = r_symbol(arg);
 
-		if (RARRAY(mem)->ptr[i] != INT2FIX(slot)) {
+		if (RARRAY(mem)->ptr[i] != ID2SYM(slot)) {
 		    rb_raise(rb_eTypeError, "struct %s not compatible (:%s for :%s)",
 			     rb_class2name(klass),
 			     rb_id2name(slot),
-			     rb_id2name(FIX2INT(RARRAY(mem)->ptr[i])));
+			     rb_id2name(SYM2ID(RARRAY(mem)->ptr[i])));
 		}
 		rb_struct_aset(v, INT2FIX(i), r_object(arg));
 	    }
@@ -878,6 +907,12 @@ r_object(arg)
 	    return r_regist(m, arg);
 	}
 
+      case TYPE_SYMBOL:
+	return ID2SYM(r_symreal(arg));
+
+      case TYPE_SYMLINK:
+	return ID2SYM(r_symlink(arg));
+
       default:
 	rb_raise(rb_eArgError, "dump format error(0x%x)", type);
 	break;
@@ -917,6 +952,7 @@ marshal_load(argc, argv)
 	GetOpenFile(port, fptr);
 	rb_io_check_readable(fptr);
 	arg.fp = fptr->f;
+	arg.taint = Qtrue;
     }
     else if (rb_respond_to(port, rb_intern("to_str"))) {
 	int len;
@@ -924,6 +960,7 @@ marshal_load(argc, argv)
 	arg.fp = 0;
 	arg.ptr = rb_str2cstr(port, &len);
 	arg.end = arg.ptr + len;
+	arg.taint = OBJ_TAINTED(port);
     }
     else {
 	rb_raise(rb_eTypeError, "instance of IO needed");
@@ -959,6 +996,4 @@ Init_marshal()
     rb_define_module_function(rb_mMarshal, "dump", marshal_dump, -1);
     rb_define_module_function(rb_mMarshal, "load", marshal_load, -1);
     rb_define_module_function(rb_mMarshal, "restore", marshal_load, -1);
-
-    rb_provide("marshal.so");	/* for backward compatibility */
 }
