@@ -1,14 +1,14 @@
-/************************************************
+/**********************************************************************
 
   numeric.c -
 
   $Author: matz $
-  $Date: 2000/07/10 08:01:32 $
+  $Date: 2000/08/31 05:29:37 $
   created at: Fri Aug 13 18:33:09 JST 1993
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
 
-************************************************/
+**********************************************************************/
 
 #include "ruby.h"
 #include <math.h>
@@ -28,10 +28,6 @@ VALUE rb_cFixnum;
 VALUE rb_eZeroDivError;
 VALUE rb_eFloatDomainError;
 
-ID rb_frame_last_func();
-VALUE rb_float_new();
-double rb_big2dbl();
-
 void
 rb_num_zerodiv()
 {
@@ -63,6 +59,7 @@ coerce_rescue(x)
 	     STR2CSTR(rb_inspect(x[1])):
 	     rb_class2name(CLASS_OF(x[1])),
 	     rb_class2name(CLASS_OF(x[0])));
+    return Qnil;		/* dummy */
 }
 
 static void
@@ -73,7 +70,8 @@ do_coerce(x, y)
     VALUE a[2];
 
     a[0] = *x; a[1] = *y;
-    ary = rb_rescue(coerce_body, (VALUE)a, coerce_rescue, (VALUE)a);
+    ary = rb_rescue2(coerce_body, (VALUE)a, coerce_rescue, (VALUE)a,
+		     rb_eStandardError, rb_eNameError, 0);
     if (TYPE(ary) != T_ARRAY || RARRAY(ary)->len != 2) {
 	rb_raise(rb_eTypeError, "coerce must return [x, y]");
     }
@@ -135,6 +133,28 @@ num_divmod(x, y)
     return rb_assoc_new(div, mod);
 }
 
+static VALUE
+num_modulo(x, y)
+    VALUE x, y;
+{
+    return rb_funcall(x, '%', 1, y);
+}
+
+static VALUE
+num_remainder(x, y)
+    VALUE x, y;
+{
+    VALUE z = rb_funcall(x, '%', 1, y);
+
+    if ((RTEST(rb_funcall(x, '<', 1, INT2FIX(0))) &&
+	 RTEST(rb_funcall(y, '>', 1, INT2FIX(0)))) ||
+	(RTEST(rb_funcall(x, '>', 1, INT2FIX(0))) &&
+	 RTEST(rb_funcall(y, '<', 1, INT2FIX(0))))) {
+	return rb_funcall(z, '-', 1, y);
+    }
+    return z;
+}
+
 static VALUE
 num_int_p(num)
     VALUE num;
@@ -298,56 +318,76 @@ flo_div(x, y)
     }
 }
 
-static VALUE
-flo_modulo(x, y, modulo)
-    VALUE x, y;
-    int modulo;
+static void
+flodivmod(x, y, divp, modp)
+    double x, y;
+    double *divp, *modp;
 {
-    double value, result;
-
-    switch (TYPE(y)) {
-      case T_FIXNUM:
-	value = (double)FIX2LONG(y);
-	break;
-      case T_BIGNUM:
-	value = rb_big2dbl(y);
-	break;
-      case T_FLOAT:
-	value = RFLOAT(y)->value;
-	break;
-      default:
-	return rb_num_coerce_bin(x, y);
-    }
+    double div, mod;
 
 #ifdef HAVE_FMOD
-    result = fmod(RFLOAT(x)->value, value);
+    mod = fmod(x, y);
 #else
     {
-	double value1 = RFLOAT(x)->value;
-	double value2;
+	double z;
 
-	modf(value1/value, &value2);
-	result = value1 - value2 * value;
+	modf(x/y, &);
+	mod = x - z * x;
     }
 #endif
-    if (modulo && value*result<0.0) {
-	result += value;
+    div = (x - mod) / y;
+    if (y*mod < 0) {
+	mod += y;
+	div -= 1.0;
     }
-    return rb_float_new(result);
+    if (modp) *modp = mod;
+    if (divp) *divp = div;
 }
 
 static VALUE
 flo_mod(x, y)
     VALUE x, y;
 {
-    return flo_modulo(x,y,1);
+    double fy, mod;
+
+    switch (TYPE(y)) {
+      case T_FIXNUM:
+	fy = (double)FIX2LONG(y);
+	break;
+      case T_BIGNUM:
+	fy = rb_big2dbl(y);
+	break;
+      case T_FLOAT:
+	fy = RFLOAT(y)->value;
+	break;
+      default:
+	return rb_num_coerce_bin(x, y);
+    }
+    flodivmod(RFLOAT(x)->value, fy, 0, &mod);
+    return rb_float_new(mod);
 }
 
 static VALUE
-flo_remainder(x, y)
+flo_divmod(x, y)
     VALUE x, y;
 {
-    return flo_modulo(x,y,0);
+    double fy, div, mod;
+
+    switch (TYPE(y)) {
+      case T_FIXNUM:
+	fy = (double)FIX2LONG(y);
+	break;
+      case T_BIGNUM:
+	fy = rb_big2dbl(y);
+	break;
+      case T_FLOAT:
+	fy = RFLOAT(y)->value;
+	break;
+      default:
+	return rb_num_coerce_bin(x, y);
+    }
+    flodivmod(RFLOAT(x)->value, fy, &div, &mod);
+    return rb_assoc_new(rb_float_new(div), rb_float_new(mod));
 }
 
 static VALUE
@@ -560,19 +600,63 @@ flo_eql(x, y)
 }
 
 static VALUE
-flo_to_i(num)
+flo_to_f(num)
     VALUE num;
 {
-    double f = RFLOAT(num)->value;
-    long val;
-
-    if (!FIXABLE(f)) {
-	return rb_dbl2big(f);
-    }
-    val = f;
-    return INT2FIX(val);
+    return num;
 }
 
+static VALUE
+flo_abs(flt)
+    VALUE flt;
+{
+    double val = fabs(RFLOAT(flt)->value);
+    return rb_float_new(val);
+}
+
+static VALUE
+flo_zero_p(num)
+    VALUE num;
+{
+    if (RFLOAT(num)->value == 0.0) {
+	return Qtrue;
+    }
+    return Qfalse;
+}
+
+static VALUE flo_is_nan_p(num)
+     VALUE num;
+{     
+
+  double value = RFLOAT(num)->value;
+
+  return isnan(value) ? Qtrue : Qfalse;
+}
+
+static VALUE flo_is_infinite_p(num)
+     VALUE num;
+{     
+  double value = RFLOAT(num)->value;
+
+  if (isinf(value)) {
+    return INT2FIX( value < 0 ? -1 : +1 );
+  }
+
+  return Qnil;
+}
+
+static VALUE flo_is_finite_p(num)
+     VALUE num;
+{     
+  double value = RFLOAT(num)->value;
+
+  if (isinf(value) || isnan(value))
+    return Qfalse;
+  
+  return Qtrue;
+}
+
+
 static VALUE
 flo_floor(num)
     VALUE num;
@@ -619,43 +703,48 @@ flo_round(num)
 }
 
 static VALUE
-flo_to_f(num)
+flo_truncate(num)
     VALUE num;
 {
-    return num;
-}
+    double f = RFLOAT(num)->value;
+    long val;
 
-static VALUE
-flo_abs(flt)
-    VALUE flt;
-{
-    double val = fabs(RFLOAT(flt)->value);
-    return rb_float_new(val);
-}
+    if (f > 0.0) f = floor(f);
+    if (f < 0.0) f = ceil(f);
 
-static VALUE
-flo_zero_p(num)
-    VALUE num;
-{
-    if (RFLOAT(num)->value == 0.0) {
-	return Qtrue;
+    if (!FIXABLE(f)) {
+	return rb_dbl2big(f);
     }
-    return Qfalse;
+    val = f;
+    return INT2FIX(val);
 }
 
 static VALUE
-to_integer(val)
-    VALUE val;
+num_floor(num)
+    VALUE num;
 {
-    return rb_funcall(val, to_i, 0);
+    return flo_floor(rb_Float(num));
 }
 
 static VALUE
-fail_to_integer(val)
-    VALUE val;
+num_ceil(num)
+    VALUE num;
 {
-    rb_raise(rb_eTypeError, "failed to convert %s into Integer",
-	     rb_class2name(CLASS_OF(val)));
+    return flo_ceil(rb_Float(num));
+}
+
+static VALUE
+num_round(num)
+    VALUE num;
+{
+    return flo_round(rb_Float(num));
+}
+
+static VALUE
+num_truncate(num)
+    VALUE num;
+{
+    return flo_truncate(rb_Float(num));
 }
 
 long
@@ -680,7 +769,7 @@ rb_num2long(val)
 
 	    sprintf(buf, "%-.10g", RFLOAT(val)->value);
 	    if (s = strchr(buf, ' ')) *s = '\0';
-	    rb_raise(rb_eTypeError, "float %s out of rang of integer", buf);
+	    rb_raise(rb_eRangeError, "float %s out of rang of integer", buf);
 	}
 
       case T_BIGNUM:
@@ -696,10 +785,7 @@ rb_num2long(val)
 	return Qnil;		/* not reached */
 
       default:
-	val = rb_rescue(to_integer, val, fail_to_integer, val);
-	if (!rb_obj_is_kind_of(val, rb_cInteger)) {
-	    rb_raise(rb_eTypeError, "`to_i' need to return integer");
-	}
+	val = rb_Integer(val);
 	return NUM2LONG(val);
     }
 }
@@ -722,7 +808,7 @@ rb_num2int(val)
     long num = rb_num2long(val);
 
     if (num < INT_MIN || INT_MAX < num) {
-	rb_raise(rb_eArgError, "integer %d too big to convert to `int'", num);
+	rb_raise(rb_eRangeError, "integer %d too big to convert to `int'", num);
     }
     return (int)num;
 }
@@ -734,7 +820,7 @@ rb_fix2int(val)
     long num = FIXNUM_P(val)?FIX2LONG(val):rb_num2long(val);
 
     if (num < INT_MIN || INT_MAX < num) {
-	rb_raise(rb_eArgError, "integer %d too big to convert to `int'", num);
+	rb_raise(rb_eRangeError, "integer %d too big to convert to `int'", num);
     }
     return (int)num;
 }
@@ -764,10 +850,17 @@ rb_num2fix(val)
 
     v = rb_num2long(val);
     if (!FIXABLE(v))
-	rb_raise(rb_eTypeError, "integer %d out of range of fixnum", v);
+	rb_raise(rb_eRangeError, "integer %d out of range of fixnum", v);
     return INT2FIX(v);
 }
 
+static VALUE
+int_to_i(num)
+    VALUE num;
+{
+    return num;
+}
+
 static VALUE
 int_int_p(num)
     VALUE num;
@@ -790,7 +883,7 @@ int_chr(num)
     long i = NUM2LONG(num);
 
     if (i < 0 || 0xff < i)
-	rb_raise(rb_eTypeError, "%d out of char range", i);
+	rb_raise(rb_eRangeError, "%d out of char range", i);
     c = i;
     return rb_str_new(&c, 1);
 }
@@ -939,37 +1032,44 @@ fix_mul(x, y)
     return rb_num_coerce_bin(x, y);
 }
 
+static void
+fixdivmod(x, y, divp, modp)
+    long x, y;
+    long *divp, *modp;
+{
+    long div, mod;
+
+    if (y == 0) rb_num_zerodiv();
+    if (y < 0) {
+	if (x < 0)
+	    div = -x / -y;
+	else
+	    div = - (x / -y);
+    }
+    else {
+	if (x < 0)
+	    div = - (-x / y);
+	else
+	    div = x / y;
+    }
+    mod = x - div*y;
+    if ((mod < 0 && y > 0) || (mod > 0 && y < 0)) {
+	mod += y;
+	div -= 1;
+    }
+    if (divp) *divp = div;
+    if (modp) *modp = mod;
+}
+
 static VALUE
 fix_div(x, y)
     VALUE x, y;
 {
     if (FIXNUM_P(y)) {
-	long i;
+	long div;
 
-	i = FIX2LONG(y);
-	if (i == 0) rb_num_zerodiv();
-	i = FIX2LONG(x)/i;
-	return INT2NUM(i);	/* FIXNUM_MIN / -1 > FIXNUM_MAX */
-    }
-    return rb_num_coerce_bin(x, y);
-}
-
-static VALUE
-fix_modulo(x, y, modulo)
-    VALUE x, y;
-{
-    long i;
-
-    if (FIXNUM_P(y)) {
-	i = FIX2LONG(y);
-	if (i == 0) rb_num_zerodiv();
-	i = FIX2LONG(x)%i;
-	if (modulo &&
-	    (FIX2LONG(x) < 0) != (FIX2LONG(y) < 0) &&
-	    i != 0) {
-	    i += FIX2LONG(y);
-	}
-	return INT2FIX(i);
+	fixdivmod(FIX2LONG(x), FIX2LONG(y), &div, 0);
+	return INT2NUM(div);
     }
     return rb_num_coerce_bin(x, y);
 }
@@ -978,14 +1078,27 @@ static VALUE
 fix_mod(x, y)
     VALUE x, y;
 {
-    return fix_modulo(x, y, 1);
+    if (FIXNUM_P(y)) {
+	long mod;
+
+	fixdivmod(FIX2LONG(x), FIX2LONG(y), 0, &mod);
+	return INT2NUM(mod);
+    }
+    return rb_num_coerce_bin(x, y);
 }
 
 static VALUE
-fix_remainder(x, y)
+fix_divmod(x, y)
     VALUE x, y;
 {
-    return fix_modulo(x, y, 0);
+    if (FIXNUM_P(y)) {
+	long div, mod;
+
+	fixdivmod(FIX2LONG(x), FIX2LONG(y), &div, &mod);
+
+	return rb_assoc_new(INT2NUM(div), INT2NUM(mod));
+    }
+    return rb_num_coerce_bin(x, y);
 }
 
 static VALUE
@@ -1192,13 +1305,6 @@ fix_aref(fix, idx)
     return INT2FIX(0);
 }
 
-static VALUE
-fix_to_i(num)
-    VALUE num;
-{
-    return num;
-}
-
 static VALUE
 fix_to_f(num)
     VALUE num;
@@ -1422,7 +1528,7 @@ Init_Numeric()
     to_i = rb_intern("to_i");
 
     rb_eZeroDivError = rb_define_class("ZeroDivisionError", rb_eStandardError);
-    rb_eFloatDomainError = rb_define_class("FloatDomainError", rb_eStandardError);
+    rb_eFloatDomainError = rb_define_class("FloatDomainError", rb_eRangeError);
     rb_cNumeric = rb_define_class("Numeric", rb_cObject);
 
     rb_include_module(rb_cNumeric, rb_mComparable);
@@ -1431,14 +1537,22 @@ Init_Numeric()
 
     rb_define_method(rb_cNumeric, "+@", num_uplus, 0);
     rb_define_method(rb_cNumeric, "-@", num_uminus, 0);
+    rb_define_method(rb_cNumeric, "===", num_equal, 1);
     rb_define_method(rb_cNumeric, "eql?", num_eql, 1);
     rb_define_method(rb_cNumeric, "divmod", num_divmod, 1);
+    rb_define_method(rb_cNumeric, "modulo", num_modulo, 1);
+    rb_define_method(rb_cNumeric, "remainder", num_remainder, 1);
     rb_define_method(rb_cNumeric, "abs", num_abs, 0);
 
     rb_define_method(rb_cNumeric, "integer?", num_int_p, 0);
     rb_define_method(rb_cNumeric, "zero?", num_zero_p, 0);
     rb_define_method(rb_cNumeric, "nonzero?", num_nonzero_p, 0);
 
+    rb_define_method(rb_cNumeric, "floor", num_floor, 0);
+    rb_define_method(rb_cNumeric, "ceil", num_ceil, 0);
+    rb_define_method(rb_cNumeric, "round", num_round, 0);
+    rb_define_method(rb_cNumeric, "truncate", num_truncate, 0);
+
     rb_cInteger = rb_define_class("Integer", rb_cNumeric);
     rb_define_method(rb_cInteger, "integer?", int_int_p, 0);
     rb_define_method(rb_cInteger, "upto", int_upto, 1);
@@ -1449,6 +1563,11 @@ Init_Numeric()
     rb_define_method(rb_cInteger, "succ", int_succ, 0);
     rb_define_method(rb_cInteger, "next", int_succ, 0);
     rb_define_method(rb_cInteger, "chr", int_chr, 0);
+    rb_define_method(rb_cInteger, "to_i", int_to_i, 0);
+    rb_define_method(rb_cInteger, "floor", int_to_i, 0);
+    rb_define_method(rb_cInteger, "ceil", int_to_i, 0);
+    rb_define_method(rb_cInteger, "round", int_to_i, 0);
+    rb_define_method(rb_cInteger, "truncate", int_to_i, 0);
 
     rb_cFixnum = rb_define_class("Fixnum", rb_cInteger);
     rb_include_module(rb_cFixnum, rb_mPrecision);
@@ -1468,7 +1587,8 @@ Init_Numeric()
     rb_define_method(rb_cFixnum, "*", fix_mul, 1);
     rb_define_method(rb_cFixnum, "/", fix_div, 1);
     rb_define_method(rb_cFixnum, "%", fix_mod, 1);
-    rb_define_method(rb_cFixnum, "remainder", fix_remainder, 1);
+    rb_define_method(rb_cFixnum, "modulo", fix_mod, 1);
+    rb_define_method(rb_cFixnum, "divmod", fix_divmod, 1);
     rb_define_method(rb_cFixnum, "**", fix_pow, 1);
 
     rb_define_method(rb_cFixnum, "abs", fix_abs, 0);
@@ -1489,7 +1609,6 @@ Init_Numeric()
     rb_define_method(rb_cFixnum, "<<", fix_lshift, 1);
     rb_define_method(rb_cFixnum, ">>", fix_rshift, 1);
 
-    rb_define_method(rb_cFixnum, "to_i", fix_to_i, 0);
     rb_define_method(rb_cFixnum, "to_f", fix_to_f, 0);
 
     rb_define_method(rb_cFixnum, "succ", fix_succ, 0);
@@ -1517,7 +1636,8 @@ Init_Numeric()
     rb_define_method(rb_cFloat, "*", flo_mul, 1);
     rb_define_method(rb_cFloat, "/", flo_div, 1);
     rb_define_method(rb_cFloat, "%", flo_mod, 1);
-    rb_define_method(rb_cFloat, "remainder", flo_remainder, 1);
+    rb_define_method(rb_cFloat, "modulo", flo_mod, 1);
+    rb_define_method(rb_cFloat, "divmod", flo_divmod, 1);
     rb_define_method(rb_cFloat, "**", flo_pow, 1);
     rb_define_method(rb_cFloat, "==", flo_eq, 1);
     rb_define_method(rb_cFloat, "<=>", flo_cmp, 1);
@@ -1527,12 +1647,17 @@ Init_Numeric()
     rb_define_method(rb_cFloat, "<=", flo_le, 1);
     rb_define_method(rb_cFloat, "eql?", flo_eql, 1);
     rb_define_method(rb_cFloat, "hash", flo_hash, 0);
-    rb_define_method(rb_cFloat, "to_i", flo_to_i, 0);
     rb_define_method(rb_cFloat, "to_f", flo_to_f, 0);
     rb_define_method(rb_cFloat, "abs", flo_abs, 0);
     rb_define_method(rb_cFloat, "zero?", flo_zero_p, 0);
 
+    rb_define_method(rb_cFloat, "to_i", flo_truncate, 0);
     rb_define_method(rb_cFloat, "floor", flo_floor, 0);
     rb_define_method(rb_cFloat, "ceil", flo_ceil, 0);
     rb_define_method(rb_cFloat, "round", flo_round, 0);
+    rb_define_method(rb_cFloat, "truncate", flo_truncate, 0);
+
+    rb_define_method(rb_cFloat, "nan?",      flo_is_nan_p, 0);
+    rb_define_method(rb_cFloat, "infinite?", flo_is_infinite_p, 0);
+    rb_define_method(rb_cFloat, "finite?",   flo_is_finite_p, 0);
 }
