@@ -1,14 +1,16 @@
-/************************************************
+/**********************************************************************
 
   process.c -
 
   $Author: matz $
-  $Date: 2000/07/24 07:16:13 $
+  $Date: 2000/08/07 05:01:54 $
   created at: Tue Aug 10 14:30:50 JST 1993
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
+  Copyright (C) 2000  Network Applied Communication Laboratory, Inc.
+  Copyright (C) 2000  Information-technology Promotion Agency, Japan
 
-************************************************/
+**********************************************************************/
 
 #include "ruby.h"
 #include "rubysig.h"
@@ -43,11 +45,6 @@ struct timeval rb_time_interval _((VALUE));
 #endif
 #include "st.h"
 
-#ifdef USE_CWGUSI
-# include <sys/errno.h>
-# include "macruby_missing.h"
-#endif
-
 #ifdef __EMX__
 #undef HAVE_GETPGRP
 #endif
@@ -96,14 +93,14 @@ rb_waitpid(pid, flags, st)
 #endif
     if (result < 0) {
 	if (errno == EINTR) {
-	    rb_thread_schedule();
+	    rb_thread_polling();
 	    goto retry;
 	}
 	return -1;
     }
     if (result == 0) {
 	if (oflags & WNOHANG) return 0;
-	rb_thread_schedule();
+	rb_thread_polling();
 	if (rb_thread_alone()) flags = oflags;
 	goto retry;
     }
@@ -160,7 +157,7 @@ wait_each(key, value, data)
 #endif
 
 static VALUE
-rb_f_wait()
+proc_wait()
 {
     int pid, state;
 #ifdef NO_WAITPID
@@ -189,13 +186,26 @@ rb_f_wait()
 }
 
 static VALUE
-rb_f_waitpid(obj, vpid, vflags)
-    VALUE obj, vpid, vflags;
+proc_wait2()
 {
+    VALUE pid = proc_wait();
+
+    return rb_assoc_new(pid, rb_last_status);
+}
+
+static VALUE
+proc_waitpid(argc, argv)
+    int argc;
+    VALUE *argv;
+{
+    VALUE vpid, vflags;
     int pid, flags, status;
 
-    if (NIL_P(vflags)) flags = 0;
-    else flags = NUM2UINT(vflags);
+    flags = 0;
+    rb_scan_args(argc, argv, "11", &vpid, &vflags);
+    if (argc == 2 && !NIL_P(vflags)) {
+	flags = NUM2UINT(vflags);
+    }
 
     if ((pid = rb_waitpid(NUM2INT(vpid), flags, &status)) < 0)
 	rb_sys_fail(0);
@@ -203,6 +213,15 @@ rb_f_waitpid(obj, vpid, vflags)
     return INT2FIX(pid);
 }
 
+static VALUE
+proc_waitpid2(argc, argv)
+    int argc;
+    VALUE *argv;
+{
+    VALUE pid = proc_waitpid2(argc, argv);
+    return rb_assoc_new(pid, rb_last_status);
+}
+
 char *strtok();
 
 #ifdef HAVE_SETITIMER
@@ -220,11 +239,9 @@ security(str)
     char *str;
 {
     if (rb_safe_level() > 0) {
-#ifndef USE_CWGUSI
 	if (rb_env_path_tainted()) {
 	    rb_raise(rb_eSecurityError, "Insecure PATH - %s", str);
 	}
-#endif
     }
 }
 
@@ -233,7 +250,6 @@ proc_exec_v(argv, prog)
     char **argv;
     char *prog;
 {
-#ifndef USE_CWGUSI
     if (prog) {
 	security(prog);
     }
@@ -286,9 +302,6 @@ proc_exec_v(argv, prog)
     execv(prog, argv);
     after_exec();
     return -1;
-#else /* USE_CWGUSI */
-    rb_notimplement();
-#endif /* USE_CWGUSI */
 }
 
 static int
@@ -319,7 +332,6 @@ int
 rb_proc_exec(str)
     const char *str;
 {
-#ifndef USE_CWGUSI
     const char *s = str;
     char *ss, *t;
     char **argv, **a;
@@ -369,9 +381,6 @@ rb_proc_exec(str)
     }
     errno = ENOENT;
     return -1;
-#else /* USE_CWGUSI */
-    rb_notimplement();
-#endif /* USE_CWGUSI */
 }
 
 #if defined(__human68k__)
@@ -506,6 +515,7 @@ rb_f_exec(argc, argv)
 	proc_exec_n(argc, argv, prog);
     }
     rb_sys_fail(RSTRING(argv[0])->ptr);
+    return Qnil;		/* dummy */
 }
 
 static VALUE
@@ -521,7 +531,7 @@ rb_f_fork(obj)
 #ifdef linux
 	after_exec();
 #endif
-	if (rb_iterator_p()) {
+	if (rb_block_given_p()) {
 	    rb_yield(Qnil);
 	    _exit(0);
 	}
@@ -555,11 +565,8 @@ rb_f_exit_bang(argc, argv, obj)
     else {
 	istatus = -1;
     }
-#ifdef USE_CWGUSI
-    exit(istatus);
-#else
     _exit(istatus);
-#endif
+
     return Qnil;		/* not reached */
 }
 
@@ -567,7 +574,7 @@ void
 rb_syswait(pid)
     int pid;
 {
-    RETSIGTYPE (*hfunc)(), (*qfunc)(), (*ifunc)();
+    RETSIGTYPE (*hfunc)_((int)), (*qfunc)_((int)), (*ifunc)_((int));
     int status;
     int i;
 
@@ -676,9 +683,6 @@ rb_f_system(argc, argv)
     }
     rb_last_status = state == -1 ? INT2FIX(127) : INT2FIX(state);
     return state == 0 ? Qtrue : Qfalse;
-#else
-#if defined(USE_CWGUSI)
-    rb_notimplement();
 #else
     volatile VALUE prog = 0;
     int pid;
@@ -731,7 +735,6 @@ rb_f_system(argc, argv)
 
     if (rb_last_status == INT2FIX(0)) return Qtrue;
     return Qfalse;
-#endif /* USE_CWGUSI */
 #endif /* __human68k__ */
 #endif /* DJGPP */
 #endif /* NT */
@@ -814,7 +817,7 @@ static VALUE
 proc_getpgid(obj, pid)
     VALUE obj, pid;
 {
-#ifdef HAVE_GETPGID
+#if defined(HAVE_GETPGID) && !defined(__CHECKER__)
     int i;
 
     i = getpgid(NUM2INT(pid));
@@ -831,6 +834,7 @@ proc_setpgid(obj, pid, pgrp)
 #ifdef HAVE_SETPGID
     int ipid, ipgrp;
 
+    rb_secure(2);
     ipid = NUM2INT(pid);
     ipgrp = NUM2INT(pgrp);
 
@@ -844,11 +848,34 @@ proc_setpgid(obj, pid, pgrp)
 static VALUE
 proc_setsid()
 {
-#ifdef HAVE_SETSID
-    int pid = setsid();
+#if defined(HAVE_SETSID)
+    int pid;
 
+    rb_secure(2);
+    pid = setsid();
     if (pid < 0) rb_sys_fail(0);
     return INT2FIX(pid);
+#elif defined(HAVE_SETPGRP) && defined(TIOCNOTTY)
+  pid_t pid;
+  int ret;
+
+  rb_secure(2);
+  pid = getpid();
+#if defined(SETPGRP_VOID)
+  ret = setpgrp();
+  /* If `pid_t setpgrp(void)' is equivalent to setsid(),
+     `ret' will be the same value as `pid', and following open() will fail.
+     In Linux, `int setpgrp(void)' is equivalent to setpgid(0, 0). */
+#else
+  ret = setpgrp(0, pid);
+#endif
+  if (ret == -1) rb_sys_fail(0);
+
+  if ((fd = open("/dev/tty", O_RDWR)) >= 0) {
+    ioctl(fd, TIOCNOTTY, NULL);
+    close(fd);
+  }
+  return INT2FIX(pid);
 #else
     rb_notimplement();
 #endif
@@ -880,6 +907,7 @@ proc_setpriority(obj, which, who, prio)
 #ifdef HAVE_GETPRIORITY
     int iwhich, iwho, iprio;
 
+    rb_secure(2);
     iwhich = NUM2INT(which);
     iwho   = NUM2INT(who);
     iprio  = NUM2INT(prio);
@@ -996,6 +1024,7 @@ static VALUE
 proc_setegid(obj, egid)
     VALUE obj, egid;
 {
+    rb_secure(2);
 #ifdef HAVE_SETEGID
     if (setegid(NUM2INT(egid)) < 0) rb_sys_fail(0);
 #else
@@ -1017,13 +1046,9 @@ VALUE rb_mProcess;
 void
 Init_process()
 {
-#ifndef USE_CWGUSI
     rb_define_virtual_variable("$$", get_pid, 0);
-#endif
     rb_define_readonly_variable("$?", &rb_last_status);
-#ifndef USE_CWGUSI
     rb_define_global_function("exec", rb_f_exec, -1);
-#endif
     rb_define_global_function("fork", rb_f_fork, 0);
     rb_define_global_function("exit!", rb_f_exit_bang, -1);
     rb_define_global_function("system", rb_f_system, -1);
@@ -1044,21 +1069,17 @@ Init_process()
 #endif
 #endif
 
-#if !defined(NT)
     rb_define_singleton_method(rb_mProcess, "fork", rb_f_fork, 0);
-#endif
     rb_define_singleton_method(rb_mProcess, "exit!", rb_f_exit_bang, -1);
-#ifndef USE_CWGUSI
     rb_define_module_function(rb_mProcess, "kill", rb_f_kill, -1);
-#endif
 #ifndef NT
-    rb_define_module_function(rb_mProcess, "wait", rb_f_wait, 0);
-    rb_define_module_function(rb_mProcess, "waitpid", rb_f_waitpid, 2);
+    rb_define_module_function(rb_mProcess, "wait", proc_wait, 0);
+    rb_define_module_function(rb_mProcess, "wait2", proc_wait2, 0);
+    rb_define_module_function(rb_mProcess, "waitpid", proc_waitpid, -1);
+    rb_define_module_function(rb_mProcess, "waitpid2", proc_waitpid2, -1);
 
-#ifndef USE_CWGUSI
     rb_define_module_function(rb_mProcess, "pid", get_pid, 0);
     rb_define_module_function(rb_mProcess, "ppid", get_ppid, 0);
-#endif /* ifndef USE_CWGUSI */
 #endif /* ifndef NT */
 
     rb_define_module_function(rb_mProcess, "getpgrp", proc_getpgrp, -1);
