@@ -1,14 +1,14 @@
-/************************************************
+/**********************************************************************
 
   random.c -
 
   $Author: matz $
-  $Date: 2000/02/08 08:53:55 $
+  $Date: 2000/08/03 09:50:33 $
   created at: Fri Dec 24 16:39:21 JST 1993
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
 
-************************************************/
+**********************************************************************/
 
 #include "ruby.h"
 
@@ -71,7 +71,7 @@ void srand48 _((long));
 
 #endif /* HAVE_RANDOM */
 
-/* 0 <= RANDOM_NUMBER <= 1 */
+/* 0 <= RANDOM_NUMBER < 1 */
 #define RANDOM_NUMBER (((double)RANDOM())/((double)RANDOM_MAX+1))
 
 #endif /* not HAVE_DRAND48 */
@@ -130,11 +130,15 @@ rb_f_srand(argc, argv, obj)
 }
 
 static VALUE
-rb_f_rand(obj, vmax)
-    VALUE obj, vmax;
+rb_f_rand(argc, argv, obj)
+    int argc;
+    VALUE *argv;
+    VALUE obj;
 {
+    VALUE vmax;
     long val, max;
 
+    rb_scan_args(argc, argv, "01", &vmax);
     if (first) {
 	struct timeval tv;
 
@@ -148,9 +152,14 @@ rb_f_rand(obj, vmax)
 	/* fall through */
       case T_BIGNUM:
 	return rb_big_rand(vmax, RANDOM_NUMBER);
+      case T_NIL:
+	max = 0;
+	break;
+      default:
+	max = NUM2LONG(vmax);
+	break;
     }
 
-    max = NUM2LONG(vmax);
     if (max == 0) {
 	return rb_float_new(RANDOM_NUMBER);
     }
@@ -164,5 +173,5 @@ void
 Init_Random()
 {
     rb_define_global_function("srand", rb_f_srand, -1);
-    rb_define_global_function("rand", rb_f_rand, 1);
+    rb_define_global_function("rand", rb_f_rand, -1);
 }
