@@ -1,14 +1,14 @@
-/************************************************
+/**********************************************************************
 
   range.c -
 
   $Author: matz $
-  $Date: 2000/01/17 08:24:10 $
+  $Date: 2000/08/31 05:29:48 $
   created at: Thu Aug 19 17:46:47 JST 1993
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
 
-************************************************/
+**********************************************************************/
 
 #include "ruby.h"
 
@@ -30,30 +30,28 @@ static VALUE
 range_failed()
 {
     rb_raise(rb_eArgError, "bad value for range");
+    return Qnil;		/* dummy */
 }
 
-static VALUE
-range_new(klass, beg, end, exclude_end)
-    VALUE klass, beg, end;
+static void
+range_init(obj, beg, end, exclude_end)
+    VALUE obj, beg, end;
     int exclude_end;
 {
-    VALUE obj;
     VALUE args[2];
 
     args[0] = beg; args[1] = end;
     if (!FIXNUM_P(beg) || !FIXNUM_P(end)) {
-	rb_rescue(range_check, (VALUE)args, range_failed, 0);
+	rb_rescue2(range_check, (VALUE)args, range_failed, 0,
+		   rb_eStandardError, rb_eNameError, 0);
     }
 
-    obj = rb_obj_alloc(klass);
     if (exclude_end) {
 	SET_EXCL(obj);
     }
 
     rb_ivar_set(obj, id_beg, beg);
     rb_ivar_set(obj, id_end, end);
-
-    return obj;
 }
 
 VALUE
@@ -61,19 +59,27 @@ rb_range_new(beg, end, exclude_end)
     VALUE beg, end;
     int exclude_end;
 {
-    return range_new(rb_cRange, beg, end, exclude_end);
+    VALUE obj = rb_obj_alloc(rb_cRange);
+
+    range_init(obj, beg, end, exclude_end);
+    return obj;
 }
 
 static VALUE
-range_s_new(argc, argv, klass)
+range_initialize(argc, argv, obj)
     int argc;
     VALUE *argv;
-    VALUE klass;
+    VALUE obj;
 {
     VALUE beg, end, flag;
     
     rb_scan_args(argc, argv, "21", &beg, &end, &flag);
-    return range_new(klass, beg, end, RTEST(flag));
+    /* Ranges are immutable, so that they should be initialized only once. */
+    if (rb_ivar_defined(obj, id_beg)) {
+	rb_raise(rb_eNameError, "`initialize' called twice");
+    }
+    range_init(obj, beg, end, RTEST(flag));
+    return Qnil;
 }
 
 static VALUE
@@ -93,12 +99,12 @@ range_eqq(range, obj)
     end = rb_ivar_get(range, id_end);
 
     if (FIXNUM_P(beg) && FIXNUM_P(obj) && FIXNUM_P(end)) {
-	if (NUM2LONG(beg) <= NUM2LONG(obj)) {
+	if (FIX2LONG(beg) <= FIX2LONG(obj)) {
 	    if (EXCL(range)) {
-		if (NUM2LONG(obj) < NUM2LONG(end)) return Qtrue;
+		if (FIX2LONG(obj) < FIX2LONG(end)) return Qtrue;
 	    }
 	    else {
-		if (NUM2LONG(obj) <= NUM2LONG(end)) return Qtrue;
+		if (FIX2LONG(obj) <= FIX2LONG(end)) return Qtrue;
 	    }
 	}
 	return Qfalse;
@@ -137,6 +143,16 @@ range_each(range)
     else if (TYPE(b) == T_STRING) {
 	rb_str_upto(b, e, EXCL(range));
     }
+    else if (rb_obj_is_kind_of(b, rb_cNumeric)) {
+	b = rb_Integer(b);
+	e = rb_Integer(e);
+
+	if (!EXCL(range)) e = rb_funcall(e, '+', 1, INT2FIX(1));
+	while (RTEST(rb_funcall(b, '<', 1, e))) {
+	    rb_yield(b);
+	    b = rb_funcall(b, '+', 1, INT2FIX(1));
+	}
+    }
     else {			/* generic each */
 	VALUE v = b;
 	ID succ = rb_intern("succ");
@@ -227,7 +243,7 @@ rb_range_beg_len(range, begp, lenp, len, err)
 
   out_of_range:
     if (err) {
-	rb_raise(rb_eIndexError, "%d..%s%d out of range",
+	rb_raise(rb_eRangeError, "%d..%s%d out of range",
 		 b, EXCL(range)?".":"", e);
     }
     return Qnil;
@@ -241,8 +257,10 @@ range_to_s(range)
 
     str = rb_obj_as_string(rb_ivar_get(range, id_beg));
     str2 = rb_obj_as_string(rb_ivar_get(range, id_end));
+    str = rb_str_dup(str);
     rb_str_cat(str, "...", EXCL(range)?3:2);
-    rb_str_cat(str, RSTRING(str2)->ptr, RSTRING(str2)->len);
+    rb_str_append(str, str2);
+    OBJ_INFECT(str, str2);
 
     return str;
 }
@@ -255,18 +273,39 @@ range_inspect(range)
 
     str = rb_inspect(rb_ivar_get(range, id_beg));
     str2 = rb_inspect(rb_ivar_get(range, id_end));
+    str = rb_str_dup(str);
     rb_str_cat(str, "...", EXCL(range)?3:2);
-    rb_str_cat(str, RSTRING(str2)->ptr, RSTRING(str2)->len);
+    rb_str_append(str, str2);
+    OBJ_INFECT(str, str2);
 
     return str;
 }
 
+static VALUE
+length_i(i, length)
+    VALUE i;
+    int *length;
+{
+    (*length)++;
+    return Qnil;
+}
+
+VALUE
+rb_length_by_each(obj)
+    VALUE obj;
+{
+    int length = 0;
+
+    rb_iterate(rb_each, obj, length_i, (VALUE)&length);
+    return INT2FIX(length);
+}
+
 static VALUE
 range_length(range)
     VALUE range;
 {
     VALUE beg, end;
-    VALUE size;
+    long size;
 
     beg = rb_ivar_get(range, id_beg);
     end = rb_ivar_get(range, id_end);
@@ -282,8 +321,8 @@ range_length(range)
 	    return INT2NUM(NUM2LONG(end) - NUM2LONG(beg) + 1);
 	}
     }
-    if (!rb_obj_is_kind_of(beg, rb_cNumeric)) {
-	return rb_enum_length(range);
+    if (!rb_obj_is_kind_of(beg, rb_cInteger)) {
+	return rb_length_by_each(range);
     }
     size = rb_funcall(end, '-', 1, beg);
     if (!EXCL(range)) {
@@ -298,7 +337,7 @@ Init_Range()
 {
     rb_cRange = rb_define_class("Range", rb_cObject);
     rb_include_module(rb_cRange, rb_mEnumerable);
-    rb_define_singleton_method(rb_cRange, "new", range_s_new, -1);
+    rb_define_method(rb_cRange, "initialize", range_initialize, -1);
     rb_define_method(rb_cRange, "===", range_eqq, 1);
     rb_define_method(rb_cRange, "each", range_each, 0);
     rb_define_method(rb_cRange, "first", range_first, 0);
