@@ -1,14 +1,14 @@
-/************************************************
+/**********************************************************************
 
   rubyio.h -
 
   $Author: matz $
-  $Date: 2000/01/17 08:24:12 $
+  $Date: 2000/05/01 09:41:29 $
   created at: Fri Nov 12 16:47:09 JST 1993
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
 
-************************************************/
+**********************************************************************/
 
 #ifndef IO_H
 #define IO_H
@@ -35,6 +35,7 @@ typedef struct OpenFile {
 #define GetOpenFile(obj,fp) rb_io_check_closed((fp) = RFILE(obj)->fptr)
 
 #define MakeOpenFile(obj, fp) do {\
+    fp = 0;\
     fp = RFILE(obj)->fptr = ALLOC(OpenFile);\
     fp->f = fp->f2 = NULL;\
     fp->mode = 0;\
@@ -58,6 +59,6 @@ void rb_io_synchronized _((OpenFile*));
 void rb_io_check_closed _((OpenFile*));
 void rb_eof_error _((void));
 
-void rb_read_check _((FILE*));	/* thread aware check */
-
+void rb_read_check _((FILE*));
+int rb_read_pending _((FILE*));
 #endif
