@@ -3,7 +3,7 @@
   rubysig.h -
 
   $Author: matz $
-  $Date: 2000/05/01 09:41:29 $
+  $Date: 2000/09/27 09:11:12 $
   created at: Wed Aug 16 01:15:38 JST 1995
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
@@ -14,8 +14,13 @@
 #define SIG_H
 
 EXTERN int rb_trap_immediate;
+#ifdef NT
+#define TRAP_BEG (rb_trap_immediate=1, SetEvent(rb_InterruptEvent))
+#define TRAP_END (rb_trap_immediate=0, ResetEvent(rb_InterruptEvent))
+#else
 #define TRAP_BEG (rb_trap_immediate=1)
 #define TRAP_END (rb_trap_immediate=0)
+#endif
 
 EXTERN int rb_prohibit_interrupt;
 #define DEFER_INTS {rb_prohibit_interrupt++;}
