@@ -3,7 +3,7 @@
   signal.c -
 
   $Author: matz $
-  $Date: 2000/09/19 07:54:22 $
+  $Date: 2000/09/27 09:11:12 $
   created at: Tue Dec 20 10:13:44 JST 1994
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
@@ -341,6 +341,14 @@ static RETSIGTYPE
 sighandle(sig)
     int sig;
 {
+#ifdef NT
+#define end_interrupt() win32_thread_resume_main()
+    if (win32_main_context(sig, sighandle)) return;
+
+#else
+#define end_interrupt() (void)0
+#endif
+
     if (sig >= NSIG) {
 	rb_bug("trap_handler: Bad signal %d", sig);
     }
@@ -358,6 +366,7 @@ sighandle(sig)
 	rb_trap_pending++;
 	trap_pending_list[sig]++;
     }
+    end_interrupt();
 }
 
 #ifdef SIGBUS
