@@ -2,8 +2,8 @@
 
   string.c -
 
-  $Author: matz $
-  $Date: 2000/09/07 06:59:36 $
+  $Author: eban $
+  $Date: 2000/09/26 09:11:47 $
   created at: Mon Aug  9 17:12:58 JST 1993
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
@@ -18,6 +18,7 @@
 #define BEG(no) regs->beg[no]
 #define END(no) regs->end[no]
 
+#include <math.h>
 #include <ctype.h>
 
 #ifdef HAVE_UNISTD_H
