@@ -1,14 +1,14 @@
-/************************************************
+/**********************************************************************
 
   util.c -
 
   $Author: matz $
-  $Date: 2000/01/17 08:24:14 $
+  $Date: 2000/07/27 09:49:24 $
   created at: Fri Mar 10 17:22:34 JST 1995
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
 
-************************************************/
+**********************************************************************/
 
 #include <stdio.h>
 
@@ -19,10 +19,6 @@
 #define RUBY_NO_INLINE
 #include "ruby.h"
 
-#ifdef USE_CWGUSI
-extern char* mktemp(char*);
-#endif
-
 VALUE
 rb_class_of(obj)
     VALUE obj;
@@ -31,6 +27,7 @@ rb_class_of(obj)
     if (obj == Qnil) return rb_cNilClass;
     if (obj == Qfalse) return rb_cFalseClass;
     if (obj == Qtrue) return rb_cTrueClass;
+    if (SYMBOL_P(obj)) return rb_cSymbol;
 
     return RBASIC(obj)->klass;
 }
@@ -43,6 +40,8 @@ rb_type(obj)
     if (obj == Qnil) return T_NIL;
     if (obj == Qfalse) return T_FALSE;
     if (obj == Qtrue) return T_TRUE;
+    if (obj == Qundef) return T_UNDEF;
+    if (SYMBOL_P(obj)) return T_SYMBOL;
 
     return BUILTIN_TYPE(obj);
 }
@@ -51,21 +50,11 @@ int
 rb_special_const_p(obj)
     VALUE obj;
 {
-    if (FIXNUM_P(obj)) return Qtrue;
-    if (obj == Qnil) return Qtrue;
-    if (obj == Qfalse) return Qtrue;
-    if (obj == Qtrue) return Qtrue;
+    if (SPECIAL_CONST_P(obj)) return Qtrue;
 
     return Qfalse;
 }
 
-int
-rb_test_false_or_nil(v)
-    VALUE v;
-{
-    return (v != Qnil) && (v != Qfalse);
-}
-
 #include "util.h"
 #ifndef HAVE_STRING_H
 char *strchr _((char*,char));
@@ -138,24 +127,6 @@ check_dir(dir)
     return dir;
 }
 
-char *
-ruby_mktemp()
-{
-    char *dir;
-    char *buf;
-
-    dir = check_dir(getenv("TMP"));
-    if (!dir) dir = check_dir(getenv("TMPDIR"));
-    if (!dir) dir = "/tmp";
-
-    buf = ALLOC_N(char,strlen(dir)+10);
-    sprintf(buf, "%s/rbXXXXXX", dir);
-    dir = mktemp(buf);
-    if (dir == NULL) free(buf);
-
-    return dir;
-}
-
 #if defined(MSDOS) || defined(__CYGWIN32__) || defined(NT)
 /*
  *  Copyright (c) 1993, Intergraph Corporation
@@ -329,7 +300,6 @@ valid_filename(char *s)
 #endif
 
 #ifdef DJGPP
-/* Copyright (C) 1995 DJ Delorie, see COPYING.DJ for details */
 #include <libc/stubs.h>
 #include <stdio.h>		/* For FILENAME_MAX */
 #include <errno.h>		/* For errno */
@@ -408,7 +378,7 @@ is_sjis1(int c)
    7. Converting all slashes to '/'
  */
 void
-_fixpath(const char *in, char *out)
+fixpath(const char *in, char *out)
 {
   int		drive_number;
   const char	*ip = in;
@@ -605,28 +575,6 @@ static void mmswap(a, b) register char *a, *b;
  }
 }
 
-static void mmswapblock(a, b, size) register char *a, *b; int size;
-{
- register int s;
- if (mmkind >= 0) {
-   register char *t = a + (size & (-16)); register int  lo = (size & 0x0C);
-   if (size >= 16) {
-     do {
-       s = A[0]; A[0] = B[0]; B[0] = s;
-       s = A[1]; A[1] = B[1]; B[1] = s;
-       s = A[2]; A[2] = B[2]; B[2] = s;
-       s = A[3]; A[3] = B[3]; B[3] = s;  a += 16; b += 16;
-     }while (a < t);
-   }
-   if (lo != 0) { s = A[0]; A[0] = B[0]; B[0] = s;
-     if (lo >= 8) { s = A[1]; A[1] = B[1]; B[1] = s;
-       if (lo == 12) {s = A[2]; A[2] = B[2]; B[2] = s;}}}
- }else{
-   register char *t = a + size;
-   do {s = *a; *a++ = *b; *b++ = s;} while (a < t);
- }
-}
-
 static void mmrot3(a, b, c) register char *a, *b, *c;
 {
  register int s;
@@ -809,3 +757,16 @@ void ruby_qsort (base, nel, size, cmp) void* base; int nel; int size; int (*cmp)
  }
 }
 
+char *
+ruby_strdup(str)
+    const char *str;
+{
+    char *tmp;
+    int len = strlen(str) + 1;
+
+    tmp = xmalloc(len);
+    if (tmp == NULL) return NULL;
+    memcpy(tmp, str, len);
+
+    return tmp;
+}
